# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'nv200widget.ui'
##
## Created by: Qt User Interface Compiler version 6.9.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractScrollArea, QAbstractSpinBox, QApplication, QCheckBox,
    QComboBox, QDoubleSpinBox, QFormLayout, QFrame,
    QGridLayout, QGroupBox, QHBoxLayout, QLabel,
    QPushButton, QScrollArea, QSizePolicy, QSpacerItem,
    QSpinBox, QSplitter, QStackedWidget, QTabWidget,
    QToolButton, QVBoxLayout, QWidget)

from pysoworks.consolewidget import Console
from pysoworks.data_recorder_widget import DataRecorderWidget
from pysoworks.mplcanvas import MplWidget
from pysoworks.nv200_controller_widget import Nv200ControllerWidget
from pysoworks.timed_progress_bar import TimedProgressBar

class Ui_NV200Widget(object):
    def setupUi(self, NV200Widget):
        if not NV200Widget.objectName():
            NV200Widget.setObjectName(u"NV200Widget")
        NV200Widget.resize(1462, 1147)
        self.verticalLayout_3 = QVBoxLayout(NV200Widget)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(9, -1, -1, -1)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(6)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.devicesComboBox = QComboBox(NV200Widget)
        self.devicesComboBox.setObjectName(u"devicesComboBox")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.devicesComboBox.sizePolicy().hasHeightForWidth())
        self.devicesComboBox.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.devicesComboBox)

        self.searchDevicesButton = QToolButton(NV200Widget)
        self.searchDevicesButton.setObjectName(u"searchDevicesButton")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.searchDevicesButton.sizePolicy().hasHeightForWidth())
        self.searchDevicesButton.setSizePolicy(sizePolicy1)
        self.searchDevicesButton.setPopupMode(QToolButton.ToolButtonPopupMode.MenuButtonPopup)
        self.searchDevicesButton.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
        self.searchDevicesButton.setProperty(u"alignedWithEdit", True)

        self.horizontalLayout.addWidget(self.searchDevicesButton)

        self.connectButton = QPushButton(NV200Widget)
        self.connectButton.setObjectName(u"connectButton")
        self.connectButton.setEnabled(False)
        sizePolicy1.setHeightForWidth(self.connectButton.sizePolicy().hasHeightForWidth())
        self.connectButton.setSizePolicy(sizePolicy1)
        self.connectButton.setProperty(u"alignedWithEdit", True)

        self.horizontalLayout.addWidget(self.connectButton)


        self.verticalLayout_3.addLayout(self.horizontalLayout)

        self.splitter = QSplitter(NV200Widget)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Orientation.Vertical)
        self.layoutWidget = QWidget(self.splitter)
        self.layoutWidget.setObjectName(u"layoutWidget")
        self.horizontalLayout_2 = QHBoxLayout(self.layoutWidget)
        self.horizontalLayout_2.setSpacing(12)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_10 = QVBoxLayout()
        self.verticalLayout_10.setObjectName(u"verticalLayout_10")
        self.verticalLayout_10.setContentsMargins(0, -1, -1, -1)
        self.scrollArea = QScrollArea(self.layoutWidget)
        self.scrollArea.setObjectName(u"scrollArea")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.scrollArea.sizePolicy().hasHeightForWidth())
        self.scrollArea.setSizePolicy(sizePolicy2)
        self.scrollArea.setMinimumSize(QSize(0, 10))
        self.scrollArea.setFrameShape(QFrame.Shape.NoFrame)
        self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.scrollArea.setSizeAdjustPolicy(QAbstractScrollArea.SizeAdjustPolicy.AdjustToContents)
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 266, 668))
        sizePolicy3 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy3.setHorizontalStretch(1)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.scrollAreaWidgetContents.sizePolicy().hasHeightForWidth())
        self.scrollAreaWidgetContents.setSizePolicy(sizePolicy3)
        self.scrollAreaWidgetContents.setStyleSheet(u"")
        self.verticalLayout_2 = QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.tabWidget = QTabWidget(self.scrollAreaWidgetContents)
        self.tabWidget.setObjectName(u"tabWidget")
        sizePolicy4 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Expanding)
        sizePolicy4.setHorizontalStretch(0)
        sizePolicy4.setVerticalStretch(0)
        sizePolicy4.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy4)
        self.tabWidget.setStyleSheet(u"")
        self.tabWidget.setTabPosition(QTabWidget.TabPosition.West)
        self.easyModeTab = QWidget()
        self.easyModeTab.setObjectName(u"easyModeTab")
        self.verticalLayout_7 = QVBoxLayout(self.easyModeTab)
        self.verticalLayout_7.setObjectName(u"verticalLayout_7")
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.easyModeGroupBox = QGroupBox(self.easyModeTab)
        self.easyModeGroupBox.setObjectName(u"easyModeGroupBox")
        self.easyModeGroupBox.setEnabled(False)
        self.verticalLayout = QVBoxLayout(self.easyModeGroupBox)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.closedLoopCheckBox = QCheckBox(self.easyModeGroupBox)
        self.closedLoopCheckBox.setObjectName(u"closedLoopCheckBox")
        self.closedLoopCheckBox.setProperty(u"toggleSwitch", True)

        self.verticalLayout.addWidget(self.closedLoopCheckBox)

        self.verticalSpacer_3 = QSpacerItem(0, 10, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)

        self.verticalLayout.addItem(self.verticalSpacer_3)

        self.targetPositionsLabel = QLabel(self.easyModeGroupBox)
        self.targetPositionsLabel.setObjectName(u"targetPositionsLabel")

        self.verticalLayout.addWidget(self.targetPositionsLabel)

        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(-1, 0, -1, -1)
        self.targetPosSpinBox_2 = QDoubleSpinBox(self.easyModeGroupBox)
        self.targetPosSpinBox_2.setObjectName(u"targetPosSpinBox_2")
        sizePolicy5 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy5.setHorizontalStretch(0)
        sizePolicy5.setVerticalStretch(0)
        sizePolicy5.setHeightForWidth(self.targetPosSpinBox_2.sizePolicy().hasHeightForWidth())
        self.targetPosSpinBox_2.setSizePolicy(sizePolicy5)
        self.targetPosSpinBox_2.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignmentFlag.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)
        self.targetPosSpinBox_2.setDecimals(3)
        self.targetPosSpinBox_2.setMaximum(1000.000000000000000)

        self.gridLayout.addWidget(self.targetPosSpinBox_2, 1, 0, 1, 1)

        self.moveButton_2 = QPushButton(self.easyModeGroupBox)
        self.moveButton_2.setObjectName(u"moveButton_2")
        sizePolicy1.setHeightForWidth(self.moveButton_2.sizePolicy().hasHeightForWidth())
        self.moveButton_2.setSizePolicy(sizePolicy1)

        self.gridLayout.addWidget(self.moveButton_2, 1, 1, 1, 1)

        self.targetPosSpinBox = QDoubleSpinBox(self.easyModeGroupBox)
        self.targetPosSpinBox.setObjectName(u"targetPosSpinBox")
        sizePolicy5.setHeightForWidth(self.targetPosSpinBox.sizePolicy().hasHeightForWidth())
        self.targetPosSpinBox.setSizePolicy(sizePolicy5)
        self.targetPosSpinBox.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignmentFlag.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)
        self.targetPosSpinBox.setDecimals(3)
        self.targetPosSpinBox.setMaximum(1000.000000000000000)

        self.gridLayout.addWidget(self.targetPosSpinBox, 0, 0, 1, 1)

        self.moveButton = QPushButton(self.easyModeGroupBox)
        self.moveButton.setObjectName(u"moveButton")
        sizePolicy1.setHeightForWidth(self.moveButton.sizePolicy().hasHeightForWidth())
        self.moveButton.setSizePolicy(sizePolicy1)

        self.gridLayout.addWidget(self.moveButton, 0, 1, 1, 1)

        self.rangeLabel = QLabel(self.easyModeGroupBox)
        self.rangeLabel.setObjectName(u"rangeLabel")
        self.rangeLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self.gridLayout.addWidget(self.rangeLabel, 2, 0, 1, 1)


        self.verticalLayout.addLayout(self.gridLayout)


        self.verticalLayout_7.addWidget(self.easyModeGroupBox)

        self.verticalSpacer = QSpacerItem(20, 740, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_7.addItem(self.verticalSpacer)

        self.tabWidget.addTab(self.easyModeTab, "")
        self.settingsTab = QWidget()
        self.settingsTab.setObjectName(u"settingsTab")
        self.verticalLayout_8 = QVBoxLayout(self.settingsTab)
        self.verticalLayout_8.setObjectName(u"verticalLayout_8")
        self.applyButton = QPushButton(self.settingsTab)
        self.applyButton.setObjectName(u"applyButton")
        self.applyButton.setStyleSheet(u"")

        self.verticalLayout_8.addWidget(self.applyButton)

        self.retrieveButton = QPushButton(self.settingsTab)
        self.retrieveButton.setObjectName(u"retrieveButton")

        self.verticalLayout_8.addWidget(self.retrieveButton)

        self.verticalSpacer_10 = QSpacerItem(20, 20, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)

        self.verticalLayout_8.addItem(self.verticalSpacer_10)

        self.restorePrevButton = QPushButton(self.settingsTab)
        self.restorePrevButton.setObjectName(u"restorePrevButton")

        self.verticalLayout_8.addWidget(self.restorePrevButton)

        self.restoreInitialButton = QPushButton(self.settingsTab)
        self.restoreInitialButton.setObjectName(u"restoreInitialButton")

        self.verticalLayout_8.addWidget(self.restoreInitialButton)

        self.restoreDefaultButton = QPushButton(self.settingsTab)
        self.restoreDefaultButton.setObjectName(u"restoreDefaultButton")

        self.verticalLayout_8.addWidget(self.restoreDefaultButton)

        self.verticalSpacer_11 = QSpacerItem(20, 20, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)

        self.verticalLayout_8.addItem(self.verticalSpacer_11)

        self.exportSettingsButton = QPushButton(self.settingsTab)
        self.exportSettingsButton.setObjectName(u"exportSettingsButton")

        self.verticalLayout_8.addWidget(self.exportSettingsButton)

        self.loadSettingsButton = QPushButton(self.settingsTab)
        self.loadSettingsButton.setObjectName(u"loadSettingsButton")

        self.verticalLayout_8.addWidget(self.loadSettingsButton)

        self.verticalSpacer_2 = QSpacerItem(20, 654, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_8.addItem(self.verticalSpacer_2)

        self.tabWidget.addTab(self.settingsTab, "")
        self.waveformTab = QWidget()
        self.waveformTab.setObjectName(u"waveformTab")
        self.verticalLayout_4 = QVBoxLayout(self.waveformTab)
        self.verticalLayout_4.setSpacing(12)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.waveformGroupBox = QGroupBox(self.waveformTab)
        self.waveformGroupBox.setObjectName(u"waveformGroupBox")
        self.formLayout = QFormLayout(self.waveformGroupBox)
        self.formLayout.setObjectName(u"formLayout")
        self.wavefomLabel = QLabel(self.waveformGroupBox)
        self.wavefomLabel.setObjectName(u"wavefomLabel")

        self.formLayout.setWidget(0, QFormLayout.ItemRole.LabelRole, self.wavefomLabel)

        self.waveFormComboBox = QComboBox(self.waveformGroupBox)
        self.waveFormComboBox.addItem("")
        self.waveFormComboBox.addItem("")
        self.waveFormComboBox.addItem("")
        self.waveFormComboBox.addItem("")
        self.waveFormComboBox.setObjectName(u"waveFormComboBox")

        self.formLayout.setWidget(0, QFormLayout.ItemRole.FieldRole, self.waveFormComboBox)

        self.verticalSpacer_6 = QSpacerItem(167, 13, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)

        self.formLayout.setItem(1, QFormLayout.ItemRole.SpanningRole, self.verticalSpacer_6)

        self.freqLabel = QLabel(self.waveformGroupBox)
        self.freqLabel.setObjectName(u"freqLabel")

        self.formLayout.setWidget(2, QFormLayout.ItemRole.LabelRole, self.freqLabel)

        self.freqSpinBox = QDoubleSpinBox(self.waveformGroupBox)
        self.freqSpinBox.setObjectName(u"freqSpinBox")
        sizePolicy5.setHeightForWidth(self.freqSpinBox.sizePolicy().hasHeightForWidth())
        self.freqSpinBox.setSizePolicy(sizePolicy5)
        self.freqSpinBox.setMinimum(0.010000000000000)
        self.freqSpinBox.setMaximum(100000.000000000000000)
        self.freqSpinBox.setValue(20.000000000000000)

        self.formLayout.setWidget(2, QFormLayout.ItemRole.FieldRole, self.freqSpinBox)

        self.waveSamplingPeriodLabel = QLabel(self.waveformGroupBox)
        self.waveSamplingPeriodLabel.setObjectName(u"waveSamplingPeriodLabel")

        self.formLayout.setWidget(3, QFormLayout.ItemRole.LabelRole, self.waveSamplingPeriodLabel)

        self.waveSamplingPeriodSpinBox = QDoubleSpinBox(self.waveformGroupBox)
        self.waveSamplingPeriodSpinBox.setObjectName(u"waveSamplingPeriodSpinBox")
        self.waveSamplingPeriodSpinBox.setReadOnly(False)
        self.waveSamplingPeriodSpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.PlusMinus)
        self.waveSamplingPeriodSpinBox.setMaximum(3276.800000000000182)
        self.waveSamplingPeriodSpinBox.setSingleStep(0.050000000000000)

        self.formLayout.setWidget(3, QFormLayout.ItemRole.FieldRole, self.waveSamplingPeriodSpinBox)

        self.phaseLabel = QLabel(self.waveformGroupBox)
        self.phaseLabel.setObjectName(u"phaseLabel")

        self.formLayout.setWidget(5, QFormLayout.ItemRole.LabelRole, self.phaseLabel)

        self.phaseShiftSpinBox = QDoubleSpinBox(self.waveformGroupBox)
        self.phaseShiftSpinBox.setObjectName(u"phaseShiftSpinBox")
        sizePolicy5.setHeightForWidth(self.phaseShiftSpinBox.sizePolicy().hasHeightForWidth())
        self.phaseShiftSpinBox.setSizePolicy(sizePolicy5)
        self.phaseShiftSpinBox.setDecimals(3)
        self.phaseShiftSpinBox.setMinimum(0.000000000000000)
        self.phaseShiftSpinBox.setMaximum(360.000000000000000)
        self.phaseShiftSpinBox.setValue(0.000000000000000)

        self.formLayout.setWidget(5, QFormLayout.ItemRole.FieldRole, self.phaseShiftSpinBox)

        self.dutyCycleLabel = QLabel(self.waveformGroupBox)
        self.dutyCycleLabel.setObjectName(u"dutyCycleLabel")

        self.formLayout.setWidget(6, QFormLayout.ItemRole.LabelRole, self.dutyCycleLabel)

        self.dutyCycleSpinBox = QDoubleSpinBox(self.waveformGroupBox)
        self.dutyCycleSpinBox.setObjectName(u"dutyCycleSpinBox")
        self.dutyCycleSpinBox.setDecimals(1)
        self.dutyCycleSpinBox.setMinimum(0.100000000000000)
        self.dutyCycleSpinBox.setValue(50.000000000000000)

        self.formLayout.setWidget(6, QFormLayout.ItemRole.FieldRole, self.dutyCycleSpinBox)

        self.verticalSpacer_5 = QSpacerItem(0, 10, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)

        self.formLayout.setItem(7, QFormLayout.ItemRole.FieldRole, self.verticalSpacer_5)

        self.highLabel = QLabel(self.waveformGroupBox)
        self.highLabel.setObjectName(u"highLabel")

        self.formLayout.setWidget(8, QFormLayout.ItemRole.LabelRole, self.highLabel)

        self.highLevelSpinBox = QDoubleSpinBox(self.waveformGroupBox)
        self.highLevelSpinBox.setObjectName(u"highLevelSpinBox")
        sizePolicy5.setHeightForWidth(self.highLevelSpinBox.sizePolicy().hasHeightForWidth())
        self.highLevelSpinBox.setSizePolicy(sizePolicy5)
        self.highLevelSpinBox.setDecimals(3)
        self.highLevelSpinBox.setMaximum(1000.000000000000000)

        self.formLayout.setWidget(8, QFormLayout.ItemRole.FieldRole, self.highLevelSpinBox)

        self.lowLabel = QLabel(self.waveformGroupBox)
        self.lowLabel.setObjectName(u"lowLabel")

        self.formLayout.setWidget(9, QFormLayout.ItemRole.LabelRole, self.lowLabel)

        self.lowLevelSpinBox = QDoubleSpinBox(self.waveformGroupBox)
        self.lowLevelSpinBox.setObjectName(u"lowLevelSpinBox")
        sizePolicy5.setHeightForWidth(self.lowLevelSpinBox.sizePolicy().hasHeightForWidth())
        self.lowLevelSpinBox.setSizePolicy(sizePolicy5)
        self.lowLevelSpinBox.setDecimals(3)
        self.lowLevelSpinBox.setMaximum(1000.000000000000000)

        self.formLayout.setWidget(9, QFormLayout.ItemRole.FieldRole, self.lowLevelSpinBox)

        self.uploadButton = QPushButton(self.waveformGroupBox)
        self.uploadButton.setObjectName(u"uploadButton")
        sizePolicy5.setHeightForWidth(self.uploadButton.sizePolicy().hasHeightForWidth())
        self.uploadButton.setSizePolicy(sizePolicy5)
        self.uploadButton.setCheckable(True)

        self.formLayout.setWidget(12, QFormLayout.ItemRole.SpanningRole, self.uploadButton)

        self.verticalSpacer_4 = QSpacerItem(20, 10, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)

        self.formLayout.setItem(11, QFormLayout.ItemRole.LabelRole, self.verticalSpacer_4)

        self.customLabel = QLabel(self.waveformGroupBox)
        self.customLabel.setObjectName(u"customLabel")

        self.formLayout.setWidget(4, QFormLayout.ItemRole.LabelRole, self.customLabel)

        self.importButton = QPushButton(self.waveformGroupBox)
        self.importButton.setObjectName(u"importButton")

        self.formLayout.setWidget(4, QFormLayout.ItemRole.FieldRole, self.importButton)


        self.verticalLayout_4.addWidget(self.waveformGroupBox)

        self.generatorGroupBox = QGroupBox(self.waveformTab)
        self.generatorGroupBox.setObjectName(u"generatorGroupBox")
        self.formLayout_2 = QFormLayout(self.generatorGroupBox)
        self.formLayout_2.setObjectName(u"formLayout_2")
        self.cyclesSpinBox = QSpinBox(self.generatorGroupBox)
        self.cyclesSpinBox.setObjectName(u"cyclesSpinBox")
        sizePolicy5.setHeightForWidth(self.cyclesSpinBox.sizePolicy().hasHeightForWidth())
        self.cyclesSpinBox.setSizePolicy(sizePolicy5)
        self.cyclesSpinBox.setMaximum(65535)
        self.cyclesSpinBox.setValue(1)

        self.formLayout_2.setWidget(0, QFormLayout.ItemRole.FieldRole, self.cyclesSpinBox)

        self.cyclesLabel = QLabel(self.generatorGroupBox)
        self.cyclesLabel.setObjectName(u"cyclesLabel")

        self.formLayout_2.setWidget(0, QFormLayout.ItemRole.LabelRole, self.cyclesLabel)

        self.totalDurationLabel = QLabel(self.generatorGroupBox)
        self.totalDurationLabel.setObjectName(u"totalDurationLabel")

        self.formLayout_2.setWidget(1, QFormLayout.ItemRole.LabelRole, self.totalDurationLabel)

        self.waveformDurationSpinBox = QSpinBox(self.generatorGroupBox)
        self.waveformDurationSpinBox.setObjectName(u"waveformDurationSpinBox")
        self.waveformDurationSpinBox.setReadOnly(True)
        self.waveformDurationSpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.NoButtons)
        self.waveformDurationSpinBox.setMaximum(16777215)

        self.formLayout_2.setWidget(1, QFormLayout.ItemRole.FieldRole, self.waveformDurationSpinBox)

        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.startWaveformButton = QPushButton(self.generatorGroupBox)
        self.startWaveformButton.setObjectName(u"startWaveformButton")
        sizePolicy5.setHeightForWidth(self.startWaveformButton.sizePolicy().hasHeightForWidth())
        self.startWaveformButton.setSizePolicy(sizePolicy5)

        self.horizontalLayout_6.addWidget(self.startWaveformButton)

        self.stopWaveformButton = QPushButton(self.generatorGroupBox)
        self.stopWaveformButton.setObjectName(u"stopWaveformButton")
        sizePolicy5.setHeightForWidth(self.stopWaveformButton.sizePolicy().hasHeightForWidth())
        self.stopWaveformButton.setSizePolicy(sizePolicy5)

        self.horizontalLayout_6.addWidget(self.stopWaveformButton)


        self.formLayout_2.setLayout(4, QFormLayout.ItemRole.SpanningRole, self.horizontalLayout_6)

        self.recSyncCheckBox = QCheckBox(self.generatorGroupBox)
        self.recSyncCheckBox.setObjectName(u"recSyncCheckBox")
        self.recSyncCheckBox.setChecked(True)
        self.recSyncCheckBox.setProperty(u"toggleSwitch", True)

        self.formLayout_2.setWidget(2, QFormLayout.ItemRole.FieldRole, self.recSyncCheckBox)

        self.recSyncLabel = QLabel(self.generatorGroupBox)
        self.recSyncLabel.setObjectName(u"recSyncLabel")

        self.formLayout_2.setWidget(2, QFormLayout.ItemRole.LabelRole, self.recSyncLabel)

        self.verticalSpacer_8 = QSpacerItem(20, 10, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)

        self.formLayout_2.setItem(3, QFormLayout.ItemRole.LabelRole, self.verticalSpacer_8)


        self.verticalLayout_4.addWidget(self.generatorGroupBox)

        self.hysteresisGroupBox = QGroupBox(self.waveformTab)
        self.hysteresisGroupBox.setObjectName(u"hysteresisGroupBox")
        self.horizontalLayout_3 = QHBoxLayout(self.hysteresisGroupBox)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.measureHysteresisButton = QPushButton(self.hysteresisGroupBox)
        self.measureHysteresisButton.setObjectName(u"measureHysteresisButton")

        self.horizontalLayout_3.addWidget(self.measureHysteresisButton)

        self.plotHysteresisButton = QPushButton(self.hysteresisGroupBox)
        self.plotHysteresisButton.setObjectName(u"plotHysteresisButton")

        self.horizontalLayout_3.addWidget(self.plotHysteresisButton)


        self.verticalLayout_4.addWidget(self.hysteresisGroupBox)

        self.verticalSpacer_7 = QSpacerItem(0, 0, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_4.addItem(self.verticalSpacer_7)

        self.tabWidget.addTab(self.waveformTab, "")
        self.resonanceTab = QWidget()
        self.resonanceTab.setObjectName(u"resonanceTab")
        self.verticalLayout_6 = QVBoxLayout(self.resonanceTab)
        self.verticalLayout_6.setObjectName(u"verticalLayout_6")
        self.resonanceButton = QPushButton(self.resonanceTab)
        self.resonanceButton.setObjectName(u"resonanceButton")

        self.verticalLayout_6.addWidget(self.resonanceButton)

        self.impulseVoltagesGroupBox = QGroupBox(self.resonanceTab)
        self.impulseVoltagesGroupBox.setObjectName(u"impulseVoltagesGroupBox")
        self.formLayout_3 = QFormLayout(self.impulseVoltagesGroupBox)
        self.formLayout_3.setObjectName(u"formLayout_3")
        self.impulsePeakVoltageLabel = QLabel(self.impulseVoltagesGroupBox)
        self.impulsePeakVoltageLabel.setObjectName(u"impulsePeakVoltageLabel")

        self.formLayout_3.setWidget(0, QFormLayout.ItemRole.LabelRole, self.impulsePeakVoltageLabel)

        self.impulsePeakVoltageSpinBox = QDoubleSpinBox(self.impulseVoltagesGroupBox)
        self.impulsePeakVoltageSpinBox.setObjectName(u"impulsePeakVoltageSpinBox")
        self.impulsePeakVoltageSpinBox.setReadOnly(True)
        self.impulsePeakVoltageSpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.NoButtons)
        self.impulsePeakVoltageSpinBox.setDecimals(1)
        self.impulsePeakVoltageSpinBox.setMinimum(-10000.000000000000000)
        self.impulsePeakVoltageSpinBox.setMaximum(10000.000000000000000)

        self.formLayout_3.setWidget(0, QFormLayout.ItemRole.FieldRole, self.impulsePeakVoltageSpinBox)

        self.impulseBaseVoltageLabel = QLabel(self.impulseVoltagesGroupBox)
        self.impulseBaseVoltageLabel.setObjectName(u"impulseBaseVoltageLabel")

        self.formLayout_3.setWidget(1, QFormLayout.ItemRole.LabelRole, self.impulseBaseVoltageLabel)

        self.impulseBaseVoltageSpinBox = QDoubleSpinBox(self.impulseVoltagesGroupBox)
        self.impulseBaseVoltageSpinBox.setObjectName(u"impulseBaseVoltageSpinBox")
        self.impulseBaseVoltageSpinBox.setReadOnly(True)
        self.impulseBaseVoltageSpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.NoButtons)
        self.impulseBaseVoltageSpinBox.setDecimals(1)
        self.impulseBaseVoltageSpinBox.setMinimum(-10000.000000000000000)
        self.impulseBaseVoltageSpinBox.setMaximum(10000.000000000000000)

        self.formLayout_3.setWidget(1, QFormLayout.ItemRole.FieldRole, self.impulseBaseVoltageSpinBox)


        self.verticalLayout_6.addWidget(self.impulseVoltagesGroupBox)

        self.verticalSpacer_9 = QSpacerItem(10, 626, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_6.addItem(self.verticalSpacer_9)

        self.tabWidget.addTab(self.resonanceTab, "")

        self.verticalLayout_2.addWidget(self.tabWidget)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.verticalLayout_10.addWidget(self.scrollArea)

        self.piezoIconLabel = QLabel(self.layoutWidget)
        self.piezoIconLabel.setObjectName(u"piezoIconLabel")
        sizePolicy5.setHeightForWidth(self.piezoIconLabel.sizePolicy().hasHeightForWidth())
        self.piezoIconLabel.setSizePolicy(sizePolicy5)
        self.piezoIconLabel.setStyleSheet(u"QLabel {margin-top: 12px; margin-left: 4px;}")

        self.verticalLayout_10.addWidget(self.piezoIconLabel)

        self.consoleButton = QToolButton(self.layoutWidget)
        self.consoleButton.setObjectName(u"consoleButton")
        sizePolicy1.setHeightForWidth(self.consoleButton.sizePolicy().hasHeightForWidth())
        self.consoleButton.setSizePolicy(sizePolicy1)
        self.consoleButton.setAutoRaise(True)

        self.verticalLayout_10.addWidget(self.consoleButton)


        self.horizontalLayout_2.addLayout(self.verticalLayout_10)

        self.stackedWidget = QStackedWidget(self.layoutWidget)
        self.stackedWidget.setObjectName(u"stackedWidget")
        sizePolicy6 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy6.setHorizontalStretch(0)
        sizePolicy6.setVerticalStretch(0)
        sizePolicy6.setHeightForWidth(self.stackedWidget.sizePolicy().hasHeightForWidth())
        self.stackedWidget.setSizePolicy(sizePolicy6)
        self.easyModePlot = MplWidget()
        self.easyModePlot.setObjectName(u"easyModePlot")
        self.stackedWidget.addWidget(self.easyModePlot)
        self.controllerStructureWidget = Nv200ControllerWidget()
        self.controllerStructureWidget.setObjectName(u"controllerStructureWidget")
        self.stackedWidget.addWidget(self.controllerStructureWidget)
        self.waveformPlots = QWidget()
        self.waveformPlots.setObjectName(u"waveformPlots")
        self.verticalLayout_9 = QVBoxLayout(self.waveformPlots)
        self.verticalLayout_9.setObjectName(u"verticalLayout_9")
        self.verticalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.splitter_2 = QSplitter(self.waveformPlots)
        self.splitter_2.setObjectName(u"splitter_2")
        self.splitter_2.setOrientation(Qt.Orientation.Vertical)
        self.waveformPlot = DataRecorderWidget(self.splitter_2)
        self.waveformPlot.setObjectName(u"waveformPlot")
        self.splitter_2.addWidget(self.waveformPlot)
        self.hysteresisPlot = MplWidget(self.splitter_2)
        self.hysteresisPlot.setObjectName(u"hysteresisPlot")
        self.splitter_2.addWidget(self.hysteresisPlot)

        self.verticalLayout_9.addWidget(self.splitter_2)

        self.stackedWidget.addWidget(self.waveformPlots)
        self.resonancePlots = QWidget()
        self.resonancePlots.setObjectName(u"resonancePlots")
        self.verticalLayout_5 = QVBoxLayout(self.resonancePlots)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.impulsePlot = MplWidget(self.resonancePlots)
        self.impulsePlot.setObjectName(u"impulsePlot")

        self.verticalLayout_5.addWidget(self.impulsePlot)

        self.resonancePlot = MplWidget(self.resonancePlots)
        self.resonancePlot.setObjectName(u"resonancePlot")

        self.verticalLayout_5.addWidget(self.resonancePlot)

        self.stackedWidget.addWidget(self.resonancePlots)

        self.horizontalLayout_2.addWidget(self.stackedWidget)

        self.splitter.addWidget(self.layoutWidget)
        self.consoleWidget = QWidget(self.splitter)
        self.consoleWidget.setObjectName(u"consoleWidget")
        sizePolicy6.setHeightForWidth(self.consoleWidget.sizePolicy().hasHeightForWidth())
        self.consoleWidget.setSizePolicy(sizePolicy6)
        self.consoleWidget.setMinimumSize(QSize(0, 0))
        self.verticalLayout_11 = QVBoxLayout(self.consoleWidget)
        self.verticalLayout_11.setObjectName(u"verticalLayout_11")
        self.verticalLayout_11.setContentsMargins(0, 0, 0, 0)
        self.consoleLabel = QLabel(self.consoleWidget)
        self.consoleLabel.setObjectName(u"consoleLabel")
        sizePolicy5.setHeightForWidth(self.consoleLabel.sizePolicy().hasHeightForWidth())
        self.consoleLabel.setSizePolicy(sizePolicy5)

        self.verticalLayout_11.addWidget(self.consoleLabel)

        self.console = Console(self.consoleWidget)
        self.console.setObjectName(u"console")
        sizePolicy6.setHeightForWidth(self.console.sizePolicy().hasHeightForWidth())
        self.console.setSizePolicy(sizePolicy6)
        self.console.setStyleSheet(u"QTextEdit { background: black; }")

        self.verticalLayout_11.addWidget(self.console)

        self.splitter.addWidget(self.consoleWidget)

        self.verticalLayout_3.addWidget(self.splitter)

        self.mainProgressBar = TimedProgressBar(NV200Widget)
        self.mainProgressBar.setObjectName(u"mainProgressBar")
        self.mainProgressBar.setMaximumSize(QSize(16777215, 3))
        self.mainProgressBar.setStyleSheet(u"QProgressBar { background: transparent;}")
        self.mainProgressBar.setValue(0)
        self.mainProgressBar.setTextVisible(False)

        self.verticalLayout_3.addWidget(self.mainProgressBar)


        self.retranslateUi(NV200Widget)

        self.tabWidget.setCurrentIndex(1)
        self.stackedWidget.setCurrentIndex(2)


        QMetaObject.connectSlotsByName(NV200Widget)
    # setupUi

    def retranslateUi(self, NV200Widget):
        NV200Widget.setWindowTitle(QCoreApplication.translate("NV200Widget", u"Form", None))
#if QT_CONFIG(tooltip)
        self.devicesComboBox.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Device List</span></p><p>List of detected NV200 devices.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        self.searchDevicesButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Search Devices</span></p><p>Search for all NV200 devices connected via USB or Ethernet. Click the menu button to search only USB or only Ethernet devices.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.searchDevicesButton.setText(QCoreApplication.translate("NV200Widget", u"Search Devices ...", None))
        self.searchDevicesButton.setProperty(u"style", QCoreApplication.translate("NV200Widget", u"pushButton", None))
#if QT_CONFIG(tooltip)
        self.connectButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Connect</span></p><p>Connect to the device you selected from the list.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.connectButton.setText(QCoreApplication.translate("NV200Widget", u"Connect", None))
        self.easyModeGroupBox.setTitle(QCoreApplication.translate("NV200Widget", u"Easy Mode", None))
#if QT_CONFIG(tooltip)
        self.closedLoopCheckBox.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Open Loop / Closed Loop</span></p><p>Toggles between open loop (OL) and closed loop (CL) mode. In open loop mode the PID-controller is bridged and the command input directly controls the amplifier. The resulting piezo stroke then depends on the characteristic of the piezo actuator and is affected by piezo-typical creeping and hysteresis behavior.</p><p>In closed loop mode (CL), these effects will be compensated by the digital loop controller.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.closedLoopCheckBox.setText(QCoreApplication.translate("NV200Widget", u"Open Loop", None))
        self.targetPositionsLabel.setText(QCoreApplication.translate("NV200Widget", u"Setpoints", None))
#if QT_CONFIG(tooltip)
        self.targetPosSpinBox_2.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Setpoint 2</span></p><p>The second setpoint. Depending on the Open Loop / Closed Loop switch this value is given as voltage (Open Loop) or as position in \u00b5m or mrad (Closed Loop).</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        self.moveButton_2.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Move To Setpoint 2</span></p><p>Starts the move to 2nd setpoint.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.moveButton_2.setText("")
#if QT_CONFIG(tooltip)
        self.targetPosSpinBox.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Setpoint 1</span></p><p>The first setpoint. Depending on the Open Loop / Closed Loop switch this value is given as voltage (Open Loop) or as position in \u00b5m or mrad (Closed Loop).</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.targetPosSpinBox.setPrefix("")
        self.targetPosSpinBox.setSuffix("")
#if QT_CONFIG(tooltip)
        self.moveButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Move To Setpoint 1 </span></p><p>Starts the move to first setpoint.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.moveButton.setText("")
        self.rangeLabel.setText("")
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.easyModeTab), QCoreApplication.translate("NV200Widget", u"Easy Mode", None))
#if QT_CONFIG(tooltip)
        self.applyButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Apply Parameters</span></p><p>Send the currently edited parameters to the device to update its configuration.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.applyButton.setText(QCoreApplication.translate("NV200Widget", u"Apply Parameters", None))
        self.applyButton.setProperty(u"text-align", QCoreApplication.translate("NV200Widget", u"left", None))
#if QT_CONFIG(tooltip)
        self.retrieveButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Retrieve Parameters</span></p><p>Read the current parameters from the device and update the local view in case you modified the parameters from outside or via a terminal program.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.retrieveButton.setText(QCoreApplication.translate("NV200Widget", u"Retrieve Parameters", None))
        self.retrieveButton.setProperty(u"text-align", QCoreApplication.translate("NV200Widget", u"left", None))
#if QT_CONFIG(tooltip)
        self.restorePrevButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Restore Previous Parameters</span></p><p>Reverts parameters to their values before the last time you clicked &quot;Apply Parameters&quot;.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.restorePrevButton.setText(QCoreApplication.translate("NV200Widget", u"Restore Previous Parameters", None))
        self.restorePrevButton.setProperty(u"text-align", QCoreApplication.translate("NV200Widget", u"left", None))
#if QT_CONFIG(tooltip)
        self.restoreInitialButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Restore Initial Parameters</span></p><p>Load the parameters as they were when the device was first connected this session.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.restoreInitialButton.setText(QCoreApplication.translate("NV200Widget", u"Restore Initial Parameters", None))
        self.restoreInitialButton.setProperty(u"text-align", QCoreApplication.translate("NV200Widget", u"left", None))
#if QT_CONFIG(tooltip)
        self.restoreDefaultButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Restore Original Parameters</span></p><p>Restore the parameters that were backed up when this device was first connected.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.restoreDefaultButton.setText(QCoreApplication.translate("NV200Widget", u"Restore Original Parameters", None))
        self.restoreDefaultButton.setProperty(u"text-align", QCoreApplication.translate("NV200Widget", u"left", None))
#if QT_CONFIG(tooltip)
        self.exportSettingsButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Export Parameters</span></p><p>Exports the current parameters into a parameters *.ini file.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.exportSettingsButton.setText(QCoreApplication.translate("NV200Widget", u"Export Parameters", None))
        self.exportSettingsButton.setProperty(u"text-align", QCoreApplication.translate("NV200Widget", u"left", None))
#if QT_CONFIG(tooltip)
        self.loadSettingsButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Load Parameters</span></p><p>Loads the parameters from a previously exported *.ini file.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.loadSettingsButton.setText(QCoreApplication.translate("NV200Widget", u"Load Parameters", None))
        self.loadSettingsButton.setProperty(u"text-align", QCoreApplication.translate("NV200Widget", u"left", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.settingsTab), QCoreApplication.translate("NV200Widget", u"Settings", None))
        self.waveformGroupBox.setTitle(QCoreApplication.translate("NV200Widget", u"Waveform Settings", None))
        self.wavefomLabel.setText(QCoreApplication.translate("NV200Widget", u"Waveform", None))
        self.waveFormComboBox.setItemText(0, QCoreApplication.translate("NV200Widget", u"Sine", None))
        self.waveFormComboBox.setItemText(1, QCoreApplication.translate("NV200Widget", u"Triangle", None))
        self.waveFormComboBox.setItemText(2, QCoreApplication.translate("NV200Widget", u"Square", None))
        self.waveFormComboBox.setItemText(3, QCoreApplication.translate("NV200Widget", u"Custom", None))

        self.freqLabel.setText(QCoreApplication.translate("NV200Widget", u"Freq.", None))
        self.freqSpinBox.setSuffix(QCoreApplication.translate("NV200Widget", u" Hz", None))
        self.waveSamplingPeriodLabel.setText(QCoreApplication.translate("NV200Widget", u"Sampling Period", None))
        self.waveSamplingPeriodSpinBox.setSuffix(QCoreApplication.translate("NV200Widget", u" ms", None))
        self.phaseLabel.setText(QCoreApplication.translate("NV200Widget", u"Phase Shift", None))
        self.phaseShiftSpinBox.setSuffix(QCoreApplication.translate("NV200Widget", u" \u00b0", None))
        self.dutyCycleLabel.setText(QCoreApplication.translate("NV200Widget", u"Duty Cycle", None))
        self.dutyCycleSpinBox.setSuffix(QCoreApplication.translate("NV200Widget", u" %", None))
        self.highLabel.setText(QCoreApplication.translate("NV200Widget", u"High Level", None))
        self.lowLabel.setText(QCoreApplication.translate("NV200Widget", u"Low Level", None))
        self.uploadButton.setText(QCoreApplication.translate("NV200Widget", u"Upload", None))
        self.customLabel.setText(QCoreApplication.translate("NV200Widget", u"Cust. Waveform", None))
        self.importButton.setText(QCoreApplication.translate("NV200Widget", u"Load CSV / Excel", None))
        self.generatorGroupBox.setTitle(QCoreApplication.translate("NV200Widget", u"Generator Control", None))
        self.cyclesLabel.setText(QCoreApplication.translate("NV200Widget", u"Cycles", None))
        self.totalDurationLabel.setText(QCoreApplication.translate("NV200Widget", u"Total Duration", None))
        self.waveformDurationSpinBox.setSpecialValueText(QCoreApplication.translate("NV200Widget", u"infinite", None))
        self.waveformDurationSpinBox.setSuffix(QCoreApplication.translate("NV200Widget", u" ms", None))
        self.startWaveformButton.setText(QCoreApplication.translate("NV200Widget", u"Start", None))
        self.stopWaveformButton.setText(QCoreApplication.translate("NV200Widget", u"Stop", None))
        self.recSyncCheckBox.setText("")
        self.recSyncLabel.setText(QCoreApplication.translate("NV200Widget", u"Sync Rec. Duration", None))
        self.hysteresisGroupBox.setTitle(QCoreApplication.translate("NV200Widget", u"Hysteresis Measurement", None))
        self.measureHysteresisButton.setText(QCoreApplication.translate("NV200Widget", u"Measure", None))
        self.plotHysteresisButton.setText(QCoreApplication.translate("NV200Widget", u"Plot", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.waveformTab), QCoreApplication.translate("NV200Widget", u"Waveform", None))
        self.resonanceButton.setText(QCoreApplication.translate("NV200Widget", u"Get Resonance Spectrum", None))
        self.impulseVoltagesGroupBox.setTitle(QCoreApplication.translate("NV200Widget", u"Impulse Voltages", None))
        self.impulsePeakVoltageLabel.setText(QCoreApplication.translate("NV200Widget", u"Peak Voltage:", None))
        self.impulsePeakVoltageSpinBox.setSuffix(QCoreApplication.translate("NV200Widget", u" V", None))
        self.impulseBaseVoltageLabel.setText(QCoreApplication.translate("NV200Widget", u"Base Voltage:", None))
        self.impulseBaseVoltageSpinBox.setSuffix(QCoreApplication.translate("NV200Widget", u" V", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.resonanceTab), QCoreApplication.translate("NV200Widget", u"Impulse Response", None))
        self.piezoIconLabel.setText(QCoreApplication.translate("NV200Widget", u"piezosystemjena Icon", None))
#if QT_CONFIG(tooltip)
        self.consoleButton.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Show / Hide Command Console</span></p><p>Toggle the command console display.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.consoleButton.setText(QCoreApplication.translate("NV200Widget", u"Console", None))
        self.consoleLabel.setText(QCoreApplication.translate("NV200Widget", u"Command Line Interface", None))
#if QT_CONFIG(tooltip)
        self.console.setToolTip(QCoreApplication.translate("NV200Widget", u"<html><head/><body><p><span style=\" font-weight:700;\">Command Console</span></p><p>Enter NV200 commands or help to show all commands</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
    # retranslateUi




from typing import Sequence, Union
from matplotlib import lines
import numpy as np
import csv
import pandas as pd
from matplotlib.backends.backend_qtagg import FigureCanvas
from matplotlib.backends.backend_qtagg import NavigationToolbar2QT
from matplotlib.colors import to_rgba
from matplotlib.axes import Axes
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from PySide6.QtCore import QStandardPaths, QTimer
from PySide6.QtGui import QPalette, QColor, QAction
from PySide6.QtWidgets import QVBoxLayout, QWidget, QFileDialog
from nv200.data_recorder import DataRecorder
from pysoworks.ui_helpers import get_icon


def mpl_color(color: QColor) -> tuple[float, float, float, float]:
    """
    Converts a QColor to a tuple of floats in the range 0.0–1.0.
    """
    return (
        color.redF(),   # R in 0.0–1.0
        color.greenF(),
        color.blueF(),
        color.alphaF()
    )


class MplCanvas(FigureCanvas):
    '''
    Class to represent the FigureCanvas widget for integration of Matplotlib with Qt.
    '''
    _fig: Figure = None

    def __init__(self, parent=None, width=5, height=4, dpi=100):
        plt.style.use('dark_background')
        self._fig = Figure(figsize=(width, height), dpi=dpi)
        self._fig.tight_layout()
        self.ax1 = self._fig.add_subplot(111)
        self.axes_list : list[Axes] = [self.ax1]
        self._fig.subplots_adjust(left=0.05, right=0.95, top=0.95, bottom=0.05)

        ax = self.ax1
        ax.set_xlabel('Time (ms)')
        ax.set_ylabel('Value')
        self.init_axes_object(ax)

        self.ax2 : Axes | None = None  # Secondary axis for two-line plots
        super().__init__(self._fig)

        self._resize_timer = QTimer(self)
        self._resize_timer.setSingleShot(True)
        self._resize_timer.timeout.connect(self.update_layout)


    def init_axes_object(self, ax : Axes):
        """
        Initializes the appearance of the given Matplotlib Axes object.
        This method configures the grid, spine colors, and tick parameters
        to use a dark gray color scheme for improved visual consistency.
        Args:
            ax (Axes): The Matplotlib Axes object to be styled.
        """
        ax.grid(True, color='darkgray', linestyle='--', linewidth=0.5)
        ax.spines['top'].set_color('darkgray')
        ax.spines['right'].set_color('darkgray')
        ax.spines['bottom'].set_color('darkgray')
        ax.spines['left'].set_color('darkgray')

        # Set tick parameters for dark grey color
        ax.tick_params(axis='x', colors='darkgray')
        ax.tick_params(axis='y', colors='darkgray')


    def get_axes(self, index : int) -> Axes:
        """
        Retrieve the matplotlib Axes object at the specified index.

        Args:
            index (int): The index of the axes to retrieve. 
                - 0: Returns the primary axes.
                - 1: Returns the secondary axes if it exists, or creates it if it does not.

        Returns:
            Axes: The requested matplotlib Axes object.

        Raises:
            IndexError: If the index is not 0 or 1.
        """
        if index < len(self.axes_list):
            return self.axes_list[index]
        
        if index == 1:
            self.ax2 = self.ax1.twinx()
            self.init_axes_object(self.ax2)
            self.axes_list.append(self.ax2)
            return self.ax2

        raise IndexError(f"Index {index} out of range for axes list.")


    def resizeEvent(self, event):
        """
        Handles the widget's resize event.

        Calls the parent class's resizeEvent to ensure default behavior,
        then updates the layout to maintain proper spacing and appearance
        after the widget has been resized.

        Args:
            event (QResizeEvent): The resize event containing the new size information.
        """
        super().resizeEvent(event)
        self._resize_timer.start(100)  # 250 ms after last resize event


    def update_layout(self):
        """
        Updates the layout of the figure to ensure proper spacing and alignment.
        This method is useful after adding or modifying elements in the figure.
        """
        new_size = self.size()
        if new_size.width() > 0 and new_size.height() > 0:  # avoid singular matrix
            self._fig.tight_layout()
            self.draw()


    def set_plot_title(self, title: str):
        """
        Sets the title of the plot.

        Args:
            title (str): The title to set for the plot.
        """
        self.ax1.set_title(title)  # Set title with dark gray color
        self.update_layout


    def plot_recorder_data(self, rec_data : DataRecorder.ChannelRecordingData, color : QColor = QColor('orange'), axis : int = 0):
        """
        Plots the data and stores the line object for later removal.
        """
        self.remove_all_axes_lines(axis)  # Remove all previous lines before plotting new data
        self.add_recorder_data_line(rec_data, color, axis)  # Add the new line to the plot


    def add_recorder_data_line(self, rec_data : DataRecorder.ChannelRecordingData, color : QColor = QColor('orange'), axis : int = 0):
        """
        Adds a new line plot to the canvas using the provided channel recording data.
        """
        self.add_line(rec_data.sample_times_ms, rec_data.values, str(rec_data.source), color, axis)  # Add the new line to the plot


    def plot_data(self, x_data: Union[Sequence[float], np.ndarray], y_data: Union[Sequence[float], np.ndarray], label: str, color : QColor = QColor('orange'), axis : int = 0):
        """
        Plots the data and stores the line object for later removal.
        """
        self.remove_all_axes_lines(axis)  # Remove all previous lines before plotting new data
        self.add_line(x_data, y_data, label, color, axis)  # Add the new line to the plot


    def add_line(self, x_data: Sequence[float], y_data: Sequence[float], label: str, color : QColor = QColor('orange'), axis : int = 0):
        """
        Adds a new line plot to the canvas 
        """
        # Plot the data and add a label for the legend
        ax = self.get_axes(axis)

        rgba = (
            color.redF(),   # R in 0.0–1.0
            color.greenF(),
            color.blueF(),
            color.alphaF()
        )
        
        print(f"Adding line with color: {rgba} and label: {label}")
        line, = ax.plot(
            x_data, y_data, 
            linestyle='-', color=rgba, label=label
        )

        ax.set_autoscale_on(True)       # Turns autoscale mode back on
        ax.set_xlim(auto=True)          # Reset x-axis limits
        ax.set_ylim(auto=True)          # Reset y-axis limits

        # Autoscale the axes after plotting the data
        ax.relim()
        ax.autoscale_view()
        
        #Show the legend with custom styling
        ax.legend(
            facecolor='darkgray', 
            edgecolor='darkgray', 
            frameon=True, 
            loc='best', 
            fontsize=10
        )

        # Redraw the canvas
        self.update_layout()


    def autoscale(self, axis: int = 0):
        """
        Automatically adjusts the axis limits to fit the plotted data.
        Parameters:
            axis (int, optional): The index of the axis to autoscale. Defaults to 0.
        This method enables autoscaling for the specified axis, resets the x and y axis limits,
        recalculates the data limits, updates the view, and redraws the canvas to reflect the changes.
        """
        ax = self.get_axes(axis)
        ax.set_autoscale_on(True)       # Turns autoscale mode back on
        ax.set_xlim(auto=True)          # Reset x-axis limits
        ax.set_ylim(auto=True)          # Reset y-axis limits

        # Autoscale the axes after plotting the data
        ax.relim()
        ax.autoscale_view()

        # Redraw the canvas
        self.update_layout()

   
    def update_line(self, line_index: int, x_data: Sequence[float], y_data: Sequence[float], axis : int = 0):
        """
        Updates the data of a specific line in the plot.
        """
        ax = self.get_axes(axis)
        lines = ax.get_lines()
        
        if 0 <= line_index < len(lines):
            line = lines[line_index]
            line.set_xdata(x_data)
            line.set_ydata(y_data)

            # Rescale the axes to fit the new data
            ax.relim()
            ax.autoscale_view()

            # Redraw the canvas to reflect the changes
            self.draw()
        else:
            raise IndexError("Line index out of range.")


    def get_line_color(self, line_index: int, axis : int = 0) -> QColor:
        """
        Returns the color of a specific line in the plot.
        """
        ax = self.get_axes(axis)
        lines = ax.get_lines()
        
        if 0 <= line_index < len(lines):
            line = lines[line_index]
            mpl_color = line.get_color()
            r, g, b, a = to_rgba(mpl_color)
            qcolor = QColor.fromRgbF(r, g, b, a)
            return qcolor
        else:
            raise IndexError("Line index out of range.")
        

    def set_line_color(self, line_index: int, color: QColor, axis : int = 0):
        """
        Sets the color of a specific line in the plot.
        """
        ax = self.get_axes(axis)
        lines = ax.get_lines()
        
        if 0 <= line_index < len(lines):
            line = lines[line_index]
            rgba = (
                color.redF(),   # R in 0.0–1.0
                color.greenF(),
                color.blueF(),
                color.alphaF()
            )
            print(f"Setting line color: {rgba}")
            line.set_color(rgba)
            self.draw()


    def get_lines(self, axis: int = 0) -> Sequence:
        """
        Returns a sequence of all lines in the plot.
        """
        ax = self.get_axes(axis)
        return ax.get_lines()
    

    def get_line_count(self, axis: int = 0) -> int:
        """
        Returns the number of lines in the plot.
        """
        ax = self.get_axes(axis)
        return len(ax.get_lines())


    def scale_axes(self, x_min: float, x_max: float, y_min: float, y_max: float, axis: int = 0):
        """
        Scales the axes to the specified limits.
        """
        ax = self.get_axes(axis)
        ax.set_xlim(x_min, x_max)
        ax.set_ylim(y_min, y_max)

        # Redraw the canvas to reflect the changes
        self.draw()    

    def remove_all_axes_lines(self, axis: int = 0):
        """Removes all lines from the axes."""
        if axis >= len(self.axes_list):
            return

        ax = self.axes_list[axis]
        # Iterate over all lines in the axes and remove them
        for line in ax.get_lines():
            line.remove()

        # Redraw the canvas to reflect the change
        self.draw()


    def clear_plot(self):
        """
        Clears the plot by removing all lines and resetting the axes.
        """
        self.remove_all_axes_lines(0)
        self.remove_all_axes_lines(1)


    def set_dark_mode(self, dark_mode: bool):
        """
        Sets the dark mode for the canvas.
        
        Args:
            dark_mode (bool): If True, sets the canvas to dark mode; otherwise, sets it to light mode.
        """
        # Define colors
        bg_color = 'black' if dark_mode else 'white'
        fg_color = 'darkgray'  # Used for ticks, labels, spines, and grid
        text_color = mpl_color(QPalette().color(QPalette.ColorRole.WindowText))
  
        # Update figure background
        self._fig.patch.set_facecolor(bg_color)

        # Update all axes in axes_list
        for ax in self.axes_list:
            ax.set_facecolor(bg_color)
            ax.tick_params(colors=fg_color)
            ax.xaxis.label.set_color(text_color)
            ax.yaxis.label.set_color(text_color)
            ax.title.set_color(text_color)

            # Update axes spines
            for spine in ax.spines.values():
                spine.set_color(fg_color)

            # Update tick label colors
            for label in ax.get_xticklabels() + ax.get_yticklabels():
                label.set_color(text_color)

        self._fig.canvas.draw_idle()  # Redraw the canvas

    
    def create_export_data(self) -> pd.DataFrame | None:
        """
        Exports the data from all lines in the first axes as a pandas DataFrame.
        The method handles two cases:
        1. If all lines have identical X data arrays and lengths, the DataFrame will have one X column and one column per line's Y data.
        2. If lines have different X data arrays or lengths, the DataFrame will use the union of all X values, aligning Y data accordingly and filling missing values with NaN (then forward-filling).
        Returns:
            pd.DataFrame | None: The exported data as a DataFrame, or None if there are no lines to export.
        """
        ax = self.axes_list[0]
        lines = ax.get_lines()

        if not lines:
            print("No data to export.")
            return

        # Gather all lines' lengths and X data
        lengths = [len(line.get_xdata()) for line in lines]
        all_lengths_equal = all(length == lengths[0] for length in lengths)

        # Check if all lines have identical X data arrays (if lengths equal)
        if all_lengths_equal:
            base_x = lines[0].get_xdata()
            all_x_same = all(np.array_equal(line.get_xdata(), base_x) for line in lines)
        else:
            all_x_same = False

        if all_lengths_equal and all_x_same:
            # Simple export: all share same x and same length
            x_data = lines[0].get_xdata()
            data = {ax.get_xlabel(): x_data}
            for i, line in enumerate(lines, start=1):
                label = line.get_label()
                if not label or label.startswith("_"):
                    label = f"y{i}"
                data[label] = line.get_ydata()

            df = pd.DataFrame(data)
        else:
            # More complicated: shared full X from first line, partial Y data per line
            
            # Create union of all X data points from all lines
            all_x_values = set()
            for line in lines:
                all_x_values.update(line.get_xdata())
            full_x = np.array(sorted(all_x_values))

            x_to_index = {x_val: idx for idx, x_val in enumerate(full_x)}
            data = {ax.get_xlabel(): full_x}

            for i, line in enumerate(lines, start=1):
                line_x = line.get_xdata()
                line_y = line.get_ydata()
                label = line.get_label()
                if not label or label.startswith("_"):
                    label = f"y{i}"

                y_full = np.full_like(full_x, fill_value=np.nan, dtype=float)

                for lx, ly in zip(line_x, line_y):
                    idx = x_to_index.get(lx)
                    if idx is not None:
                        y_full[idx] = ly
                    else:
                        print(f"Warning: x value {lx} in line '{label}' not found in full X data")

                data[label] = y_full

            df = pd.DataFrame(data)
            # Forward-fill missing Y values down each column except 'x'
            df.loc[:, df.columns != "x"] = df.loc[:, df.columns != "x"].ffill()
        
        return df
    
   
    def export_plot_data(self) -> None:
        """
        Export the data from the first Matplotlib Axes in the widget's canvas to CSV or Excel.

        This function:
        - Assumes the widget has an attribute `canvas` (a Matplotlib FigureCanvas).
        - Retrieves all Line2D objects from the first axes in the figure.
        - Exports the X values and each line's Y values into a DataFrame.
        - Prompts the user to select a save location and format (*.csv or *.xlsx).
        - Automatically saves in the chosen format, defaulting to CSV if no extension is given.

        File format behavior:
        - *.csv   → saved with `DataFrame.to_csv(index=False)`
        - *.xlsx  → saved with `DataFrame.to_excel(index=False)` (requires openpyxl)

        Dependencies:
        pip install pandas openpyxl

        Raises:
        AttributeError: If `self.canvas` is missing or is not a Matplotlib FigureCanvas.
        """
        df = self.create_export_data()
        if df is None:
            return

        home_dir = QStandardPaths.writableLocation(QStandardPaths.HomeLocation)
        file_path, selected_filter = QFileDialog.getSaveFileName(
            self,
            "Save Data",
            home_dir,
            "CSV Files (*.csv);;Excel Files (*.xlsx)",
            "Excel Files (*.xlsx)"
        )

        if not file_path:
            return

        if "csv" in selected_filter.lower():
            if not file_path.lower().endswith(".csv"):
                file_path += ".csv"
            df.to_csv(file_path, index=False)
            print(f"Data exported to CSV: {file_path}")

        elif "xlsx" in selected_filter.lower():
            if not file_path.lower().endswith(".xlsx"):
                file_path += ".xlsx"
            df.to_excel(file_path, index=False)
            print(f"Data exported to Excel: {file_path}")


class LightIconToolbar(NavigationToolbar2QT):
    """
    A customized Matplotlib navigation toolbar for Qt applications with light-themed icons.
    This toolbar extends the default NavigationToolbar2QT to provide:
    - Custom icons for standard navigation actions (home, back, forward, pan, zoom, save, etc.)
    - A custom "Clear Plot" action with its own icon.
    - Icon initialization on first show event to ensure proper styling.
    """
    _icons_initialized : bool = False

    def __init__(self, canvas, parent):
        super().__init__(canvas, parent)


    def add_custom_action(self, action: QAction, index : int = -1):
        """
        Adds a custom action to the toolbar at the specified index.
        
        Args:
            index (int): The position to insert the action. Defaults to -1 (end of the toolbar).
            action (QAction): The action to add. If None, a default action is created.
        """
        self.insertAction(self.actions()[index], action)
            

    def showEvent(self, event):
        super().showEvent(event)
        if not self._icons_initialized:
            self._initialize_icons()
            self._icons_initialized = True

    def _initialize_icons(self):
        icon_paths = {
            'home': 'home',
            'back': 'arrow_back',
            'forward': 'arrow_forward',
            'pan': 'pan_tool',
            'zoom': 'zoom_in',
            'save_figure': 'file_save',
            #'configure_subplots': 'line_axis',
            'configure_subplots': '-',
            'edit_parameters': 'tune',
        }

        for action_name, icon_path in icon_paths.items():
            action = self._actions.get(action_name)
            if action:
                if icon_path == "-":
                    # Remove action if icon path is "-"
                    self.removeAction(action)
                else:
                    icon = get_icon(
                        icon_path,
                        size=24,
                        fill=False,
                        color=QPalette.ColorRole.WindowText
                    )
                    action.setIcon(icon)

    def set_dark_mode(self, dark_mode: bool):
        """
        Sets the dark mode for the toolbar.
        
        Args:
            dark_mode (bool): If True, sets the toolbar to dark mode; otherwise, sets it to light mode.
        """
        self._initialize_icons() 


class MplWidget(QWidget):
    '''
    Widget promoted and defined in Qt Designer
    '''
    def __init__(self, parent = None):
        QWidget.__init__(self, parent)
        self.canvas = MplCanvas(self)
        # Create the navigation toolbar linked to the canvas
        self.toolbar = LightIconToolbar(self.canvas, self)
        self.export_action : QAction | None = None
        self.vbl = QVBoxLayout()
        self.vbl.addWidget(self.toolbar)
        self.vbl.addWidget(self.canvas)
        self.vbl.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.vbl)
        self.setContentsMargins(0, 0, 0, 0)


    def add_toolbar_action(self, action: QAction):
        """
        Adds a custom action to the toolbar.
        
        Args:
            action (QAction): The action to add to the toolbar.
        """
        self.toolbar.add_custom_action(action)

    def show_export_action(self):
        """
        Shows the export to CSV action.
        """
        if self.export_action:
            return
        self.export_action = a = QAction(get_icon('export_notes', size=24, fill=False, color=QPalette.ColorRole.WindowText), "Export to Excel / CSV", self)
        self.add_toolbar_action(a)
        a.triggered.connect(self.canvas.export_plot_data)

    def add_toolbar_separator(self):
        """
        Adds a separator to the toolbar.
        """
        action = QAction(self)
        action.setSeparator(True)
        self.toolbar.add_custom_action(action)


    def set_dark_mode(self, dark_mode: bool):
        """
        Sets the dark mode for the canvas and toolbar.
        
        Args:
            dark_mode (bool): If True, sets the canvas and toolbar to dark mode; otherwise, sets them to light mode.
        """
        self.canvas.set_dark_mode(dark_mode)
        self.toolbar.set_dark_mode(dark_mode)


    



from PySide6.QtWidgets import QTextEdit
from PySide6.QtCore import Signal, Qt
from PySide6.QtGui import QKeyEvent, QTextCursor, QTextOption, QFont, QTextCharFormat, QColor


class Console(QTextEdit):
    """
    A custom console widget that emulates a command-line interface.

    Features:
    - Accepts user input and emits a signal when Enter is pressed.
    - Maintains a history of entered commands and allows navigation via arrow keys.
    - Displays output from external sources (e.g. a remote device).
    """

    command_entered: Signal = Signal(str)

    def __init__(self, parent=None) -> None:
        """
        Initializes the console widget.
        """
        super().__init__(parent)
        self.setAcceptRichText(False)
        self.setUndoRedoEnabled(False)
        self.setWordWrapMode(QTextOption.WrapMode.WrapAnywhere)

        # Set fixed-width font
        font = QFont("Consolas")  # or use "Monospace" / "Courier" depending on platform
        font.setStyleHint(QFont.StyleHint.Monospace)
        font.setFixedPitch(True)
        font.setPointSize(10)  # Set to desired size
        self.setFont(font)

        self.history: list[str] = []
        self.history_index: int = -1
        self.prompt: str = ">>> "
        self.prompt_count = 0  # Count of prompts inserted
        self.insert_prompt()  # Ensure prompt is present on initialization
        self.command_help : dict[str, str] = {}


    def _ensure_prompt_present(self) -> None:
        """
        Ensures that the prompt is present at the end of the console.
        This is used after command processing, in case the handler did not insert it.
        """
        if not self.prompt_count:
            self.insert_prompt()

    def insert_prompt(self) -> None:
        """
        Inserts a new command prompt at the end of the console.
        """
        cursor = self.textCursor()
        cursor.movePosition(QTextCursor.MoveOperation.End)
        if not self.document().isEmpty():
            cursor.insertText("\n") 
        cursor.insertText(self.prompt)
        self.move_cursor_to_end()
        self.prompt_count += 1


    def show_help(self, command: str) -> None:
        """
        Displays help information for available commands.
        If no specific command is provided, lists all commands with their help descriptions.
        If one or more commands are specified (comma-separated), displays help for each specified command.
        If a command is not recognized, notifies that no help is available for that command.
        Args:
            command (str): The input string containing 'help' and optionally a comma-separated list of commands.
        Returns:
            None
        """
        parts = command.split(',', 1)  # Split into ['help', 'cl,set'] or just ['help']
        
        output_lines = []
        if len(parts) == 1 or not parts[1].strip():
            # No parameters given, print all commands with help
            for cmd, help_text in self.command_help.items():
                output_lines.append(f"{cmd}\t{help_text}")
        else:
            # Parameters given, split by commas and print help for each
            cmds = [c.strip().lower() for c in parts[1].split(',')]
            for cmd in cmds:
                help_text = self.command_help.get(cmd)
                if help_text:
                    output_lines.append(f"{cmd}: {help_text}")
                else:
                    output_lines.append(f"No help available for '{cmd}'.")
        
        self.print_output('\n'.join(output_lines))
        return


    def execute_command(self) -> None:
        """
        Extracts and executes the current command entered in the console widget.
        Strips off the prompt (e.g. >>>) and emits `command_entered`.
        Adds the command to the history and ensures the prompt is reset.
        """
        cursor = self.textCursor()
        block = cursor.block()
        line_text = block.text()
        command = line_text[len(self.prompt.lstrip('\n\r\t')):].strip()

        self.move_cursor_to_end()

        if command:
            self.history.append(command)
            self.history_index = len(self.history)

        if command.lower() == "cls":
            self.clear_console()
            return
        
        if command.lower().startswith("help"):
            self.show_help(command)
            return

        self.prompt_count = 0  # Reset prompt count after command entry
        self.command_entered.emit(command)
        self._ensure_prompt_present()  # Ensure prompt is present after command processing



    def keyPressEvent(self, event: QKeyEvent) -> None:
        """
        Handles key events for Enter, Up, and Down arrows.
        Emits a signal when Enter is pressed with the input command.
        """
        cursor = self.textCursor()

        if event.key() in (Qt.Key_Return, Qt.Key_Enter):
            self.execute_command()

        elif event.key() == Qt.Key_Up:
            if self.history and self.history_index > 0:
                self.history_index -= 1
                self.replace_current_line(self.history[self.history_index])

        elif event.key() == Qt.Key_Down:
            if self.history and self.history_index < len(self.history) - 1:
                self.history_index += 1
                self.replace_current_line(self.history[self.history_index])
            elif self.history_index == len(self.history) - 1:
                self.history_index += 1
                self.replace_current_line("")

        elif event.key() == Qt.Key_Backspace:
            # Prevent backspace if cursor is before or within the prompt
            if cursor.positionInBlock() <= len(self.prompt):
                return  # Ignore the backspace
            else:
                super().keyPressEvent(event)

        else:
            super().keyPressEvent(event)

    def replace_current_line(self, text: str) -> None:
        """
        Replaces the current input line with the given text.
        Used for navigating through command history.

        :param text: The text to replace the current line with.
        """
        cursor = self.textCursor()
        block = cursor.block()  # Get the current text block (line)
        
        cursor.beginEditBlock()  # Begin grouped undo action

        # Select the entire block (line)
        cursor.setPosition(block.position())
        cursor.movePosition(QTextCursor.EndOfBlock, QTextCursor.KeepAnchor)

        # Replace line text
        cursor.removeSelectedText()
        cursor.insertText(self.prompt + text)
        cursor.endEditBlock()

    def move_cursor_to_end(self) -> None:
        """
        Moves the text cursor to the end of the document.
        """
        cursor: QTextCursor = self.textCursor()
        cursor.movePosition(QTextCursor.End)
        self.setTextCursor(cursor)

    def print_output(self, text: str) -> None:
        """
        Appends output to the console. If the text starts with 'error',
        it is printed in red.
        """
        # Insert formatted text
        if not text:
            self.insert_prompt()
            return
        
        cursor = self.textCursor()
        cursor.movePosition(QTextCursor.End)
        cursor.beginEditBlock()

        # Determine formatting
        fmt = QTextCharFormat()
        if text.lower().startswith("error"):
            fmt.setForeground(QColor("red"))
        else:
             fmt.setForeground(QColor("#aaaaaa"))  # Default color

        cursor.insertText("\n" + text, fmt)
        fmt.setForeground(QColor("white"))
        cursor.insertText("\n", fmt)
        cursor.endEditBlock()

        # Optionally scroll to the bottom
        self.moveCursor(QTextCursor.End)
        
        # Insert prompt if needed
        self.insert_prompt()
    
    def clear_console(self) -> None:
        """
        Clears the console output and inserts a new prompt.

        This method removes all existing text from the console widget and displays a fresh prompt for user input.
        """
        self.clear()
        self.insert_prompt()

    def register_commands(self, help_dict: dict[str, str]) -> None:
        """
        Registers a dictionary mapping command names to help messages.
        """
        self.command_help = help_dict


import traceback
from functools import wraps
from typing import Any, Callable, Awaitable
from pathlib import Path
from qt_material_icons import MaterialIcon
from PySide6.QtCore import QObject
from PySide6.QtGui import QPalette, QIcon, QPixmap, QAction
from PySide6.QtWidgets import QComboBox, QMessageBox, QStyle, QWidget
import qtinter


def get_icon(icon_name: str, size: int = 24, fill: bool = True, color : QPalette.ColorRole = QPalette.ColorRole.Highlight) -> MaterialIcon:
    """
    Creates and returns a MaterialIcon object with the specified icon name, size, fill style, and color.

    Args:
        icon_name (str): The name of the icon to retrieve.
        size (int, optional): The size of the icon in pixels. Defaults to 24.
        fill (bool, optional): Whether the icon should be filled or outlined. Defaults to True.
        color (QPalette.ColorRole, optional): The color role to use for the icon. Defaults to QPalette.ColorRole.Highlight.
    """
    icon = MaterialIcon(icon_name, size=size, fill=fill)
    icon.set_color(QPalette().color(color))
    return icon


def get_icon_for_menu(icon_name: str, size: int = 24, fill: bool = True, color : QPalette.ColorRole = QPalette.ColorRole.Highlight) -> MaterialIcon:
    """
    Returns the icon of the given QMenu, if it has one.

    Args:
        menu (QMenu): The QMenu instance to retrieve the icon from.

    Returns:
        QIcon | None: The icon of the menu, or None if not set.
    """
    icon = get_icon(icon_name, size=size, fill=fill, color=color)
    icon.set_color(QPalette().color(QPalette.ColorRole.Window), QIcon.Mode.Active)
    return icon


def set_combobox_index_by_value(combo: QComboBox, value: Any) -> None:
    """
    Sets the current index of the QComboBox to the item with the given userData value.

    Args:
        combo: The QComboBox instance.
        value: The value to match in userData of combo items.
    """
    index: int = combo.findData(value)
    if index != -1:
        combo.setCurrentIndex(index)
    else:
        raise ValueError(f"Value {value!r} not found in QComboBox.")
    

def images_path() -> Path:
    """
    Returns the absolute path to the images directory within the current module.

    This function constructs the path based on the location of this file and returns it as a Path object.
    """
    base_dir = Path(__file__).parent
    return base_dir / "assets" / "images"


def safe_asyncslot(coro_func: Callable[..., Awaitable], handler=None):
    """
    Wrap qtinter.asyncslot to catch exceptions from async slots.

    Example:
        >>> ui.searchDevicesButton.clicked.connect(safe_asyncslot(self.search_all_devices))
    """
    if handler is None:
        handler = default_exception_handler

    @wraps(coro_func)
    async def wrapper(*args, **kwargs):
        try:
            return await coro_func(*args, **kwargs)
        except Exception as e:
            handler(e)

    return qtinter.asyncslot(wrapper)


def default_exception_handler(exc: BaseException):
    """
    Handles uncaught exceptions by printing the traceback to the console and displaying a critical error message box.

    Args:
        exc (BaseException): The exception instance to handle.

    Side Effects:
        - Prints the full traceback of the exception to the standard output.
        - Displays a critical QMessageBox with the exception message.
    """
    tb = ''.join(traceback.format_exception(type(exc), exc, exc.__traceback__))
    print("Unhandled Exception:\n", tb)
    QMessageBox.critical(None, "Unhandled Exception", str(exc))


def repolish(w : QWidget):
    """
    Calls unpolish() / polish for the style of the given widget to update
    stylesheet if a property changes
    """
    w.style().unpolish(w)
    w.style().polish(w)


def company_logo_pixmap(dark_theme : bool) -> QPixmap :
    """
    Returns a QPixmap object containing the company logo image, selecting the appropriate version for dark or light themes.

    Args:
        dark_theme (bool): If True, returns the logo suitable for dark themes; otherwise, returns the logo for light themes.

    Returns:
        QPixmap: The pixmap containing the company logo image.
    """
    if dark_theme:
        image_file = "piezosystem_logo_white@2x.png"
    else:
        image_file = "piezosystem_logo@2x.png"
    image_path = images_path() / image_file
    return QPixmap(str(image_path))


def menu_separator(parent : QObject | None) -> QAction:
    """
    Creates and returns a QAction that serves as a separator in menus.

    Returns:
        QAction: A QAction configured as a separator.
    """
    sep = QAction(parent)
    sep.setSeparator(True)
    return sep



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'data_recorder_widget.ui'
##
## Created by: Qt User Interface Compiler version 6.9.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractSpinBox, QApplication, QComboBox, QDoubleSpinBox,
    QFrame, QGridLayout, QGroupBox, QLabel,
    QLayout, QSizePolicy, QSpacerItem, QSpinBox,
    QVBoxLayout, QWidget)

from pysoworks.mplcanvas import MplWidget

class Ui_DataRecorderWidget(object):
    def setupUi(self, DataRecorderWidget):
        if not DataRecorderWidget.objectName():
            DataRecorderWidget.setObjectName(u"DataRecorderWidget")
        DataRecorderWidget.resize(1071, 774)
        self.verticalLayout = QVBoxLayout(DataRecorderWidget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.dataRecSettingsGroupBox = QGroupBox(DataRecorderWidget)
        self.dataRecSettingsGroupBox.setObjectName(u"dataRecSettingsGroupBox")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dataRecSettingsGroupBox.sizePolicy().hasHeightForWidth())
        self.dataRecSettingsGroupBox.setSizePolicy(sizePolicy)
        self.gridLayout = QGridLayout(self.dataRecSettingsGroupBox)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setSizeConstraint(QLayout.SizeConstraint.SetDefaultConstraint)
        self.gridLayout.setContentsMargins(-1, -1, -1, 9)
        self.recsrc1ComboBox = QComboBox(self.dataRecSettingsGroupBox)
        self.recsrc1ComboBox.setObjectName(u"recsrc1ComboBox")

        self.gridLayout.addWidget(self.recsrc1ComboBox, 0, 1, 1, 1)

        self.samplePeriodSpinBox = QDoubleSpinBox(self.dataRecSettingsGroupBox)
        self.samplePeriodSpinBox.setObjectName(u"samplePeriodSpinBox")
        sizePolicy.setHeightForWidth(self.samplePeriodSpinBox.sizePolicy().hasHeightForWidth())
        self.samplePeriodSpinBox.setSizePolicy(sizePolicy)
        self.samplePeriodSpinBox.setReadOnly(True)
        self.samplePeriodSpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.NoButtons)
        self.samplePeriodSpinBox.setMaximum(16777215.000000000000000)

        self.gridLayout.addWidget(self.samplePeriodSpinBox, 1, 3, 1, 1)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer, 0, 4, 1, 1)

        self.recDurationSpinBox = QSpinBox(self.dataRecSettingsGroupBox)
        self.recDurationSpinBox.setObjectName(u"recDurationSpinBox")
        sizePolicy.setHeightForWidth(self.recDurationSpinBox.sizePolicy().hasHeightForWidth())
        self.recDurationSpinBox.setSizePolicy(sizePolicy)
        self.recDurationSpinBox.setMinimum(1)
        self.recDurationSpinBox.setMaximum(16777215)

        self.gridLayout.addWidget(self.recDurationSpinBox, 0, 3, 1, 1)

        self.recDurationLabel = QLabel(self.dataRecSettingsGroupBox)
        self.recDurationLabel.setObjectName(u"recDurationLabel")
        self.recDurationLabel.setStyleSheet(u"margin-left: 6px")

        self.gridLayout.addWidget(self.recDurationLabel, 0, 2, 1, 1)

        self.channel1Label = QLabel(self.dataRecSettingsGroupBox)
        self.channel1Label.setObjectName(u"channel1Label")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.channel1Label.sizePolicy().hasHeightForWidth())
        self.channel1Label.setSizePolicy(sizePolicy1)

        self.gridLayout.addWidget(self.channel1Label, 0, 0, 1, 1)

        self.channekl2Label = QLabel(self.dataRecSettingsGroupBox)
        self.channekl2Label.setObjectName(u"channekl2Label")
        sizePolicy1.setHeightForWidth(self.channekl2Label.sizePolicy().hasHeightForWidth())
        self.channekl2Label.setSizePolicy(sizePolicy1)
        self.channekl2Label.setStyleSheet(u"")

        self.gridLayout.addWidget(self.channekl2Label, 1, 0, 1, 1)

        self.recsrc2ComboBox = QComboBox(self.dataRecSettingsGroupBox)
        self.recsrc2ComboBox.setObjectName(u"recsrc2ComboBox")

        self.gridLayout.addWidget(self.recsrc2ComboBox, 1, 1, 1, 1)

        self.samplingPeriodLabel = QLabel(self.dataRecSettingsGroupBox)
        self.samplingPeriodLabel.setObjectName(u"samplingPeriodLabel")
        self.samplingPeriodLabel.setStyleSheet(u"margin-left: 6px")

        self.gridLayout.addWidget(self.samplingPeriodLabel, 1, 2, 1, 1)


        self.verticalLayout.addWidget(self.dataRecSettingsGroupBox)

        self.mplWidget = MplWidget(DataRecorderWidget)
        self.mplWidget.setObjectName(u"mplWidget")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.mplWidget.sizePolicy().hasHeightForWidth())
        self.mplWidget.setSizePolicy(sizePolicy2)
        self.mplWidget.setStyleSheet(u"background: lightgrey")

        self.verticalLayout.addWidget(self.mplWidget)


        self.retranslateUi(DataRecorderWidget)

        QMetaObject.connectSlotsByName(DataRecorderWidget)
    # setupUi

    def retranslateUi(self, DataRecorderWidget):
        DataRecorderWidget.setWindowTitle(QCoreApplication.translate("DataRecorderWidget", u"Frame", None))
        self.dataRecSettingsGroupBox.setTitle(QCoreApplication.translate("DataRecorderWidget", u"Data Recorder", None))
#if QT_CONFIG(tooltip)
        self.recsrc1ComboBox.setToolTip(QCoreApplication.translate("DataRecorderWidget", u"<html><head/><body><p><span style=\" font-weight:700;\">Recorder Channel 1</span></p><p>Choose the source for high-frequency data (up to 20 kHz) recorded by device channel 1.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        self.samplePeriodSpinBox.setToolTip(QCoreApplication.translate("DataRecorderWidget", u"<html><head/><body><p><span style=\" font-weight:700;\">Sampling Period</span></p><p>Calculated based on the total recording duration and available memory.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.samplePeriodSpinBox.setSuffix(QCoreApplication.translate("DataRecorderWidget", u" ms", None))
#if QT_CONFIG(tooltip)
        self.recDurationSpinBox.setToolTip(QCoreApplication.translate("DataRecorderWidget", u"<html><head/><body><p><span style=\" font-weight:700;\">Recording Duration</span></p><p>Set the total duration for the internal data recorder. Longer durations result in lower sampling frequencies due to memory limits.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.recDurationSpinBox.setSuffix(QCoreApplication.translate("DataRecorderWidget", u" ms", None))
        self.recDurationLabel.setText(QCoreApplication.translate("DataRecorderWidget", u"Duration:", None))
        self.channel1Label.setText(QCoreApplication.translate("DataRecorderWidget", u"Channel 1:", None))
        self.channekl2Label.setText(QCoreApplication.translate("DataRecorderWidget", u"Channel 2:", None))
#if QT_CONFIG(tooltip)
        self.recsrc2ComboBox.setToolTip(QCoreApplication.translate("DataRecorderWidget", u"<html><head/><body><p><span style=\" font-weight:700;\">Recorder Channel 2</span></p><p>Choose the source for high-frequency data (up to 20 kHz) recorded by device channel 2.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.samplingPeriodLabel.setText(QCoreApplication.translate("DataRecorderWidget", u"Sampling Period", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'spiboxwidget.ui'
##
## Created by: Qt User Interface Compiler version 6.9.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractSpinBox, QApplication, QComboBox, QDoubleSpinBox,
    QFrame, QGridLayout, QGroupBox, QHBoxLayout,
    QLabel, QPushButton, QSizePolicy, QSpacerItem,
    QToolButton, QVBoxLayout, QWidget)

from pysoworks.mplcanvas import MplWidget
from pysoworks.timed_progress_bar import TimedProgressBar

class Ui_SpiBoxWidget(object):
    def setupUi(self, SpiBoxWidget):
        if not SpiBoxWidget.objectName():
            SpiBoxWidget.setObjectName(u"SpiBoxWidget")
        SpiBoxWidget.resize(1205, 821)
        self.verticalLayout_3 = QVBoxLayout(SpiBoxWidget)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(6)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.devicesComboBox = QComboBox(SpiBoxWidget)
        self.devicesComboBox.setObjectName(u"devicesComboBox")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.devicesComboBox.sizePolicy().hasHeightForWidth())
        self.devicesComboBox.setSizePolicy(sizePolicy)

        self.horizontalLayout.addWidget(self.devicesComboBox)

        self.searchDevicesButton = QToolButton(SpiBoxWidget)
        self.searchDevicesButton.setObjectName(u"searchDevicesButton")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.searchDevicesButton.sizePolicy().hasHeightForWidth())
        self.searchDevicesButton.setSizePolicy(sizePolicy1)
        self.searchDevicesButton.setPopupMode(QToolButton.ToolButtonPopupMode.MenuButtonPopup)
        self.searchDevicesButton.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
        self.searchDevicesButton.setProperty(u"alignedWithEdit", True)

        self.horizontalLayout.addWidget(self.searchDevicesButton)

        self.connectButton = QPushButton(SpiBoxWidget)
        self.connectButton.setObjectName(u"connectButton")
        self.connectButton.setEnabled(False)
        sizePolicy1.setHeightForWidth(self.connectButton.sizePolicy().hasHeightForWidth())
        self.connectButton.setSizePolicy(sizePolicy1)

        self.horizontalLayout.addWidget(self.connectButton)


        self.verticalLayout_3.addLayout(self.horizontalLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(12)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(-1, -1, 0, -1)
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.singleDatasetGroupBox = QGroupBox(SpiBoxWidget)
        self.singleDatasetGroupBox.setObjectName(u"singleDatasetGroupBox")
        self.singleDatasetGroupBox.setEnabled(True)
        self.singleDatasetGroupBox.setMinimumSize(QSize(0, 0))
        self.gridLayout = QGridLayout(self.singleDatasetGroupBox)
        self.gridLayout.setObjectName(u"gridLayout")
        self.singleDatasetSendCh1SpinBox = QDoubleSpinBox(self.singleDatasetGroupBox)
        self.singleDatasetSendCh1SpinBox.setObjectName(u"singleDatasetSendCh1SpinBox")
        self.singleDatasetSendCh1SpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.UpDownArrows)
        self.singleDatasetSendCh1SpinBox.setDecimals(3)
        self.singleDatasetSendCh1SpinBox.setMaximum(100.000000000000000)
        self.singleDatasetSendCh1SpinBox.setSingleStep(10.000000000000000)

        self.gridLayout.addWidget(self.singleDatasetSendCh1SpinBox, 1, 1, 1, 1)

        self.singleDatasetReceiveCh1SpinBox = QDoubleSpinBox(self.singleDatasetGroupBox)
        self.singleDatasetReceiveCh1SpinBox.setObjectName(u"singleDatasetReceiveCh1SpinBox")
        self.singleDatasetReceiveCh1SpinBox.setReadOnly(True)
        self.singleDatasetReceiveCh1SpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.NoButtons)
        self.singleDatasetReceiveCh1SpinBox.setDecimals(3)
        self.singleDatasetReceiveCh1SpinBox.setMaximum(1000.000000000000000)

        self.gridLayout.addWidget(self.singleDatasetReceiveCh1SpinBox, 1, 3, 1, 1)

        self.label_3 = QLabel(self.singleDatasetGroupBox)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self.gridLayout.addWidget(self.label_3, 0, 1, 1, 1)

        self.label_4 = QLabel(self.singleDatasetGroupBox)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self.gridLayout.addWidget(self.label_4, 0, 3, 1, 1)

        self.label = QLabel(self.singleDatasetGroupBox)
        self.label.setObjectName(u"label")

        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)

        self.label_2 = QLabel(self.singleDatasetGroupBox)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)

        self.singleDatasetReceiveCh3SpinBox = QDoubleSpinBox(self.singleDatasetGroupBox)
        self.singleDatasetReceiveCh3SpinBox.setObjectName(u"singleDatasetReceiveCh3SpinBox")
        self.singleDatasetReceiveCh3SpinBox.setReadOnly(True)
        self.singleDatasetReceiveCh3SpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.NoButtons)
        self.singleDatasetReceiveCh3SpinBox.setDecimals(3)
        self.singleDatasetReceiveCh3SpinBox.setMaximum(1000.000000000000000)

        self.gridLayout.addWidget(self.singleDatasetReceiveCh3SpinBox, 3, 3, 1, 1)

        self.singleDatasetReceiveCh2SpinBox = QDoubleSpinBox(self.singleDatasetGroupBox)
        self.singleDatasetReceiveCh2SpinBox.setObjectName(u"singleDatasetReceiveCh2SpinBox")
        self.singleDatasetReceiveCh2SpinBox.setReadOnly(True)
        self.singleDatasetReceiveCh2SpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.NoButtons)
        self.singleDatasetReceiveCh2SpinBox.setDecimals(3)
        self.singleDatasetReceiveCh2SpinBox.setMaximum(1000.000000000000000)

        self.gridLayout.addWidget(self.singleDatasetReceiveCh2SpinBox, 2, 3, 1, 1)

        self.singleDatasetSendCh3SpinBox = QDoubleSpinBox(self.singleDatasetGroupBox)
        self.singleDatasetSendCh3SpinBox.setObjectName(u"singleDatasetSendCh3SpinBox")
        self.singleDatasetSendCh3SpinBox.setReadOnly(False)
        self.singleDatasetSendCh3SpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.UpDownArrows)
        self.singleDatasetSendCh3SpinBox.setDecimals(3)
        self.singleDatasetSendCh3SpinBox.setMaximum(100.000000000000000)
        self.singleDatasetSendCh3SpinBox.setSingleStep(10.000000000000000)

        self.gridLayout.addWidget(self.singleDatasetSendCh3SpinBox, 3, 1, 1, 1)

        self.label_5 = QLabel(self.singleDatasetGroupBox)
        self.label_5.setObjectName(u"label_5")

        self.gridLayout.addWidget(self.label_5, 3, 0, 1, 1)

        self.sendSingleButton = QPushButton(self.singleDatasetGroupBox)
        self.sendSingleButton.setObjectName(u"sendSingleButton")

        self.gridLayout.addWidget(self.sendSingleButton, 4, 1, 1, 3)

        self.line = QFrame(self.singleDatasetGroupBox)
        self.line.setObjectName(u"line")
        self.line.setMinimumSize(QSize(0, 0))
        self.line.setFrameShape(QFrame.Shape.VLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line, 1, 2, 3, 1)

        self.singleDatasetSendCh2SpinBox = QDoubleSpinBox(self.singleDatasetGroupBox)
        self.singleDatasetSendCh2SpinBox.setObjectName(u"singleDatasetSendCh2SpinBox")
        self.singleDatasetSendCh2SpinBox.setButtonSymbols(QAbstractSpinBox.ButtonSymbols.UpDownArrows)
        self.singleDatasetSendCh2SpinBox.setDecimals(3)
        self.singleDatasetSendCh2SpinBox.setMaximum(100.000000000000000)
        self.singleDatasetSendCh2SpinBox.setSingleStep(10.000000000000000)

        self.gridLayout.addWidget(self.singleDatasetSendCh2SpinBox, 2, 1, 1, 1)


        self.verticalLayout_2.addWidget(self.singleDatasetGroupBox)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_2.addItem(self.verticalSpacer)


        self.horizontalLayout_2.addLayout(self.verticalLayout_2)

        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.verticalLayout_4.setContentsMargins(-1, 0, -1, -1)
        self.waveformPlot = MplWidget(SpiBoxWidget)
        self.waveformPlot.setObjectName(u"waveformPlot")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.waveformPlot.sizePolicy().hasHeightForWidth())
        self.waveformPlot.setSizePolicy(sizePolicy2)

        self.verticalLayout_4.addWidget(self.waveformPlot)


        self.horizontalLayout_2.addLayout(self.verticalLayout_4)


        self.verticalLayout_3.addLayout(self.horizontalLayout_2)

        self.moveProgressBar = TimedProgressBar(SpiBoxWidget)
        self.moveProgressBar.setObjectName(u"moveProgressBar")
        self.moveProgressBar.setMaximumSize(QSize(16777215, 3))
        self.moveProgressBar.setStyleSheet(u"QProgressBar { background: transparent;}")
        self.moveProgressBar.setValue(0)
        self.moveProgressBar.setTextVisible(False)

        self.verticalLayout_3.addWidget(self.moveProgressBar)


        self.retranslateUi(SpiBoxWidget)

        QMetaObject.connectSlotsByName(SpiBoxWidget)
    # setupUi

    def retranslateUi(self, SpiBoxWidget):
        SpiBoxWidget.setWindowTitle(QCoreApplication.translate("SpiBoxWidget", u"Form", None))
        self.searchDevicesButton.setText(QCoreApplication.translate("SpiBoxWidget", u"Search Devices ...", None))
        self.searchDevicesButton.setProperty(u"style", QCoreApplication.translate("SpiBoxWidget", u"pushButton", None))
        self.connectButton.setText(QCoreApplication.translate("SpiBoxWidget", u"Connect", None))
        self.singleDatasetGroupBox.setTitle(QCoreApplication.translate("SpiBoxWidget", u"Single Dataset", None))
        self.singleDatasetSendCh1SpinBox.setSuffix(QCoreApplication.translate("SpiBoxWidget", u" %", None))
        self.singleDatasetReceiveCh1SpinBox.setSuffix(QCoreApplication.translate("SpiBoxWidget", u" %", None))
        self.label_3.setText(QCoreApplication.translate("SpiBoxWidget", u"Send", None))
        self.label_4.setText(QCoreApplication.translate("SpiBoxWidget", u"Receive", None))
        self.label.setText(QCoreApplication.translate("SpiBoxWidget", u"Channel 1:", None))
        self.label_2.setText(QCoreApplication.translate("SpiBoxWidget", u"Channel 2:", None))
        self.singleDatasetReceiveCh3SpinBox.setSuffix(QCoreApplication.translate("SpiBoxWidget", u" %", None))
        self.singleDatasetReceiveCh2SpinBox.setSuffix(QCoreApplication.translate("SpiBoxWidget", u" %", None))
        self.singleDatasetSendCh3SpinBox.setSuffix(QCoreApplication.translate("SpiBoxWidget", u" %", None))
        self.label_5.setText(QCoreApplication.translate("SpiBoxWidget", u"Channel 3:", None))
        self.sendSingleButton.setText(QCoreApplication.translate("SpiBoxWidget", u"Send", None))
        self.singleDatasetSendCh2SpinBox.setSuffix(QCoreApplication.translate("SpiBoxWidget", u" %", None))
    # retranslateUi




__all__ = [
    'get',
    'load_text'
]

import os
from typing import Union, List
from pathlib import Path
from PySide6.QtCore import QDir


ASSET_PATH = os.path.dirname(__file__)

QDir.addSearchPath('stylesheets', os.path.join(ASSET_PATH, 'stylesheets'))


def get(name: Union[str, Path, List[str]]) -> Path:
    if isinstance(name, list):
        name = os.path.join(*name)

    outpath = os.path.join(ASSET_PATH, name)
    if os.path.commonpath([outpath, ASSET_PATH]) != ASSET_PATH:
        raise RuntimeError("Directory traversal while reading an asset")
    return Path(outpath)


def load_text(name: Union[str, List[str]]) -> str:
    with open(get(name), 'r') as f:
        return f.read()



def load_stylesheet(name: str) -> str:
    if not name.endswith('.qss'):
        name += '.qss'
    return load_text(['stylesheets', name])



import threading
import os
from typing import Optional, Type
from PySide6.QtCore import QSettings, QStandardPaths


# Get the recommended config directory for the user
config_dir: str = QStandardPaths.writableLocation(QStandardPaths.StandardLocation.AppConfigLocation)

# Ensure the directory exists
os.makedirs(config_dir, exist_ok=True)

# Define the full path to your settings INI file
ini_file_path: str = os.path.join(config_dir, "settings.ini")
print(f"INI file path: {ini_file_path}")


_settings: Optional[QSettings] = None
_settings_lock: threading.RLock = threading.RLock()



def get_settings() -> QSettings:
    """
    Lazily create and return the global QSettings instance.

    Must be called after QCoreApplication is initialized and
    application/org names are set.

    Returns:
        QSettings: The global QSettings instance using INI format and
                   application-specific config path.
    """
    global _settings
    if _settings is None:
        # Retrieve the app config path (now with correct app/org info)
        config_dir = QStandardPaths.writableLocation(QStandardPaths.StandardLocation.AppConfigLocation)
        os.makedirs(config_dir, exist_ok=True)
        ini_file_path = os.path.join(config_dir, "settings.ini")
        print(f"Creating QSettings with INI file path: {ini_file_path}")
        _settings = QSettings(ini_file_path, QSettings.Format.IniFormat)
    return _settings



class SettingsContext:
    """
    Thread-safe context manager providing exclusive access
    to the lazily initialized global QSettings instance.
    """

    def __enter__(self) -> QSettings:
        _settings_lock.acquire()
        return get_settings()

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[object],
    ) -> bool:
        try:
            get_settings().sync()
        finally:
            _settings_lock.release()
        return False




# This Python file uses the following encoding: utf-8
import sys
import asyncio
import logging
import os

from PySide6.QtWidgets import QApplication, QWidget, QMenu
from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QPalette, QIcon, QAction
import qtinter
from matplotlib.backends.backend_qtagg import FigureCanvas
from qt_material_icons import MaterialIcon

from nv200.shared_types import DetectedDevice, DiscoverFlags
from nv200.device_discovery import discover_devices
from nv200.spibox_device import SpiBoxDevice
from nv200.connection_utils import connect_to_detected_device
from pysoworks.style_manager import StyleManager, style_manager
from pysoworks.ui_helpers import get_icon, get_icon_for_menu, set_combobox_index_by_value, safe_asyncslot

# Important:
# You need to run the following command to generate the ui_form.py file
#     pyside6-uic form.ui -o ui_form.py, or
#     pyside2-uic form.ui -o ui_form.py
from pysoworks.ui_spiboxwidget import Ui_SpiBoxWidget




class SpiBoxWidget(QWidget):
    """
    Main application window for the PySoWorks UI, providing asynchronous device discovery, connection, and control features.
    Attributes:
        _device (DeviceClient): The currently connected device client, or None if not connected.
        _recorder (DataRecorder): The data recorder associated with the connected device, or None if not initialized
    """

    _device: SpiBoxDevice = None
    status_message = Signal(str, int)  # message text, timeout in ms

    def __init__(self, parent=None):
        super().__init__(parent)

        self._discover_flags : DiscoverFlags = DiscoverFlags.ALL
        self.ui = Ui_SpiBoxWidget()
        ui = self.ui
        ui.setupUi(self)
        self.init_device_search_ui()
        ui.singleDatasetGroupBox.setEnabled(False)
        ui.sendSingleButton.clicked.connect(qtinter.asyncslot(self.send_single_dataset))
        style_manager.style.dark_mode_changed.connect(ui.waveformPlot.set_dark_mode)


    def init_device_search_ui(self):
        """
        Initializes the device search UI components, including buttons and combo boxes for device selection.
        """
        ui = self.ui
        ui.searchDevicesButton.setIcon(get_icon("search", size=24, fill=True))
        ui.searchDevicesButton.clicked.connect(safe_asyncslot(self.search_all_devices))

        # Create the menu
        menu = QMenu(self)

        # Create actions
        serial_action = QAction("USB Devices", ui.searchDevicesButton)
        serial_action.setIcon(get_icon_for_menu("usb"))
        ethernet_action = QAction("Ethernet Devices", ui.searchDevicesButton)
        ethernet_action.setIcon(get_icon("lan"))

        # Connect actions to appropriate slots
        serial_action.triggered.connect(safe_asyncslot(self.search_serial_devices))
        ethernet_action.triggered.connect(safe_asyncslot(self.search_ethernet_devices))

        # Add actions to menu
        menu.addAction(serial_action)
        menu.addAction(ethernet_action)

        # Set the menu to the button
        ui.searchDevicesButton.setMenu(menu)
                                       
        ui.connectButton.clicked.connect(qtinter.asyncslot(self.connect_to_device))
        ui.connectButton.setIcon(get_icon("power", size=24, fill=True))
        ui.searchDevicesButton.clicked.connect(qtinter.asyncslot(self.search_devices))
        ui.devicesComboBox.currentIndexChanged.connect(self.on_device_selected)


    async def search_all_devices(self):
        """
        Asynchronously searches for all available devices and updates the UI accordingly.
        This method is a wrapper around search_devices to allow for easy integration with other async tasks.
        """
        self._discover_flags = DiscoverFlags.ALL
        await self.search_devices()


    async def search_serial_devices(self):
        """
        Asynchronously searches for serial devices and updates the UI accordingly.
        This method is a wrapper around search_devices to allow for easy integration with other async tasks.
        """
        self._discover_flags = DiscoverFlags.DETECT_SERIAL
        await self.search_devices()

    async def search_ethernet_devices(self):
        """
        Asynchronously searches for Ethernet devices and updates the UI accordingly.
        This method is a wrapper around search_devices to allow for easy integration with other async tasks.
        """
        self._discover_flags = DiscoverFlags.DETECT_ETHERNET
        await self.search_devices()


    async def search_devices(self):
        """
        Asynchronously searches for available devices and updates the UI accordingly.
        """
        ui = self.ui
        ui.searchDevicesButton.setEnabled(False)
        ui.connectButton.setEnabled(False)
        ui.singleDatasetGroupBox.setEnabled(False)
        self.status_message.emit("Searching for devices...", 0)
        QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)

        if self._device is not None:
            await self._device.close()
            self._device = None
        
        print("Searching...")
        ui.moveProgressBar.start(300)
        try:
            print("Discovering devices...")
            devices = await discover_devices(flags=self._discover_flags | DiscoverFlags.ADJUST_COMM_PARAMS, device_class=SpiBoxDevice)    
            
            if not devices:
                print("No devices found.")
            else:
                print(f"Found {len(devices)} device(s):")
                for device in devices:
                    print(device)
            ui.moveProgressBar.stop(success=True, context="search_devices")
        except Exception as e:
            ui.moveProgressBar.reset()
            print(f"Error: {e}")
        finally:
            QApplication.restoreOverrideCursor()
            self.ui.searchDevicesButton.setEnabled(True)
            self.status_message.emit("", 0)
            print("Search completed.")
            self.ui.devicesComboBox.clear()
            if devices:
                for device in devices:
                    self.ui.devicesComboBox.addItem(f"{device}", device)
            else:
                self.ui.devicesComboBox.addItem("No devices found.")
            
            
    def on_device_selected(self, index):
        """
        Handles the event when a device is selected from the devicesComboBox.
        """
        if index == -1:
            print("No device selected.")
            return

        device = self.ui.devicesComboBox.itemData(index, role=Qt.UserRole)
        if device is None:
            print("No device data found.")
            return
        
        print(f"Selected device: {device}")
        self.ui.connectButton.setEnabled(True)

    
    def selected_device(self) -> DetectedDevice:
        """
        Returns the currently selected device from the devicesComboBox.
        """
        index = self.ui.devicesComboBox.currentIndex()
        if index == -1:
            return None
        return self.ui.devicesComboBox.itemData(index, role=Qt.UserRole)
    
    
    async def disconnect_from_device(self):
        """
        Asynchronously disconnects from the currently connected device.
        """
        if self._device is None:
            print("No device connected.")
            return

        await self._device.close()
        self._device = None       
            


    async def connect_to_device(self):
        """
        Asynchronously connects to the selected device.
        """
        self.setCursor(Qt.WaitCursor)
        detected_device = self.selected_device()
        self.status_message.emit(f"Connecting to {detected_device.identifier}...", 0)
        print(f"Connecting to {detected_device.identifier}...")
        try:
            await self.disconnect_from_device()
            self._device = await connect_to_detected_device(detected_device)
            self.ui.singleDatasetGroupBox.setEnabled(True)
            self.status_message.emit(f"Connected to {detected_device.identifier}.", 2000)
            print(f"Connected to {detected_device.identifier}.")
        except Exception as e:
            self.status_message.emit(f"Connection failed: {e}", 2000)
            print(f"Connection failed: {e}")
            return
        finally:
            self.setCursor(Qt.ArrowCursor)


    async def send_single_dataset(self):
        """
        Handles the event when the send single button is clicked.
        Sends a single dataset to the connected SPI Box device and updates the UI with the response.
        """
        rxdata = await self._device.set_setpoints_percent(
            self.ui.singleDatasetSendCh1SpinBox.value(),
            self.ui.singleDatasetSendCh2SpinBox.value(),
            self.ui.singleDatasetSendCh3SpinBox.value()
        )

        self.ui.singleDatasetReceiveCh1SpinBox.setValue(rxdata[0])
        self.ui.singleDatasetReceiveCh2SpinBox.setValue(rxdata[1])
        self.ui.singleDatasetReceiveCh3SpinBox.setValue(rxdata[2])

    def cleanup(self):
        """
        Cleans up resources by initiating an asynchronous disconnection from the device.
        This function needs to get called, before the widget is deleted
        """
        result = asyncio.create_task(self.disconnect_from_device())


# about_dialog.py
from PySide6.QtWidgets import QDialog, QLayout, QApplication
from PySide6.QtGui import QPixmap, QIcon, QPalette
from PySide6.QtCore import QDate, Qt, QSize, QUrl
import PySide6
import sys
import pysoworks.ui_helpers as ui_helpers

from pysoworks.ui_about_dialog import Ui_AboutDialog
from pysoworks.style_manager import style_manager

class AboutDialog(QDialog):
    """
    A classical About dialog to display application information such as
    application name, version, organization name, logos, and license info.
    """

    def __init__(self, parent=None) -> None:
        """
        Initialize the About dialog.

        Args:
            parent (QWidget | None): Parent widget.
            app (QApplication | None): Reference to the application instance
                which should provide the following attributes:
                - applicationName()
                - applicationVersion()
                - organizationName()
                - organizationDomain()
                - thirdPartyLicensesUrl()
                - licenseManager.activeLicenseKey().versionTypeString()
        """
        super().__init__(parent)
        self.ui: Ui_AboutDialog = Ui_AboutDialog()
        self.ui.setupUi(self)

        # Set default logos
        self._setup_default_logos()

        # Populate labels with app info
        self._populate_info()

        # Ensure dialog has fixed size
        if self.layout() is not None:
            self.layout().setSizeConstraint(QLayout.SizeConstraint.SetFixedSize)

        ui = self.ui
        ui.urlLabel.setTextFormat(Qt.TextFormat.RichText)
        ui.urlLabel.setOpenExternalLinks(True)
        ui.urlLabel.linkActivated.connect(lambda: print("URL clicked"))

    def _setup_default_logos(self) -> None:
        """Set the default application and organization logos."""
        app_icon = QIcon(str(ui_helpers.images_path() / "app_icon_brand.svg"))
        org_pixmap = ui_helpers.company_logo_pixmap(style_manager.dark_mode)

        self.ui.applicationLogoLabel.setPixmap(app_icon.pixmap(QSize(192, 192)))
        self.ui.organizationLogoLabel.setPixmap(org_pixmap)

    def _populate_info(self) -> None:
        """Populate all labels with information from the application."""
        app = QApplication.instance()
        if app is None:
            return
        
        ui = self.ui
        
        ui.applicationLabel.setText(app.applicationName())

        # Version info
        ui.versionLabel.setText(f"Version: {app.applicationVersion()}")
        ui.qtVersionLabel.setText(f"Based on PySide: {PySide6.__version__}")


        # Organization info
        ui.urlLabel.setText(f"<a href=\"{app.organizationDomain()}\">{app.organizationDomain()}</a>")
        ui.urlLabel.setTextFormat(Qt.TextFormat.RichText)
        ui.urlLabel.setOpenExternalLinks(True)
        current_year = QDate.currentDate().toString("yyyy")
        ui.copyrightLabel.setText(
            f"© Copyright {current_year} {app.organizationName()}. All rights reserved."
        )

    # Properties for logos
    def set_application_logo(self, logo: QPixmap) -> None:
        """Set the application logo."""
        self.ui.applicationLogoLabel.setPixmap(logo)

    def application_logo(self) -> QPixmap | None:
        """Return the current application logo."""
        return self.ui.applicationLogoLabel.pixmap()

    def set_organization_logo(self, logo: QPixmap) -> None:
        """Set the organization logo."""
        self.ui.organizationLogoLabel.setPixmap(logo)

    def organization_logo(self) -> QPixmap | None:
        """Return the current organization logo."""
        return self.ui.organizationLogoLabel.pixmap()


# This Python file uses the following encoding: utf-8
import sys
import logging
import os


from PySide6.QtWidgets import QApplication, QMainWindow
from PySide6.QtCore import (
    Qt,
    QtMsgType,
    qInstallMessageHandler,
    QLocale,
    QUrl
)
from PySide6.QtGui import QIcon, QGuiApplication, QAction, QDesktopServices, QPalette

import qtinter
from pathlib import Path
import PySide6QtAds as QtAds
from rich.traceback import install as install_rich_traceback
from rich.logging import RichHandler

from pysoworks.nv200widget import NV200Widget
from pysoworks.spiboxwidget import SpiBoxWidget
from pysoworks.action_manager import ActionManager, MenuID, action_manager
from pysoworks.style_manager import StyleManager, style_manager
from pysoworks.settings_manager import SettingsContext
from pysoworks.about_dialog import AboutDialog
import pysoworks.ui_helpers as ui_helpers


def qt_message_handler(mode, context, message):
    if mode == QtMsgType.QtDebugMsg:
        print(f"[QtDebug] {message}")
    elif mode == QtMsgType.QtInfoMsg:
        print(f"[QtInfo] {message}")
    elif mode == QtMsgType.QtWarningMsg:
        print(f"[QtWarning] {message}")
    elif mode == QtMsgType.QtCriticalMsg:
        print(f"[QtCritical] {message}")
    elif mode == QtMsgType.QtFatalMsg:
        print(f"[QtFatal] {message}")

qInstallMessageHandler(qt_message_handler)


# Important:
# You need to run the following command to generate the ui_form.py file
#     pyside6-uic form.ui -o ui_form.py, or
#     pyside2-uic form.ui -o ui_form.py
from pysoworks.ui_mainwindow import Ui_MainWindow



class MainWindow(QMainWindow):
    """
    Main application window for the PySoWorks UI, providing asynchronous device discovery, connection, and control features.
    Attributes:
        _device (DeviceClient): The currently connected device client, or None if not connected.
        _recorder (DataRecorder): The data recorder associated with the connected device, or None if not initialized
    """
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_MainWindow()
        ui = self.ui
        ui.setupUi(self)

        self.init_action_manager() 

        # Create the dock manager. Because the parent parameter is a QMainWindow
        # the dock manager registers itself as the central widget.
        #QtAds.CDockManager.setConfigFlag(QtAds.CDockManager.FocusHighlighting, True)
        QtAds.CDockManager.setAutoHideConfigFlags(QtAds.CDockManager.DefaultAutoHideConfig)
        self.dock_manager = QtAds.CDockManager(self)
        self.dock_manager.setStyleSheet("")
        ui.actionNv200View.triggered.connect(self.add_nv200_view)
        ui.actionNv200View.setIcon(ui_helpers.get_icon_for_menu("add", size=24, fill=False))
        ui.actionSpiBoxView.triggered.connect(self.add_spibox_view)
        ui.actionSpiBoxView.setIcon(ui_helpers.get_icon_for_menu("add", size=24, fill=False))
        self.add_nv200_view()
        self.resize(1600, 900)  # Set initial size to 800x600

        self.init_style_controls()


    def init_action_manager(self):
        """
        Initializes the ActionManager and registers the main window.
        This method should be called after the main window is set up.
        """
        action_manager.register_main_window(self)

        # Register menus
        action_manager.register_menu(MenuID.FILE, self.ui.menuFile)
        action_manager.register_menu(MenuID.VIEW, self.ui.menuView)
        action_manager.register_menu(MenuID.HELP, self.ui.menuHelp)

        manual_action = QAction("Manual...", parent=self)
        manual_action.triggered.connect(self.show_manual)
        manual_action.setIcon(ui_helpers.get_icon_for_menu("book_2", size=24, fill=False))
        action_manager.add_action_to_menu(MenuID.HELP, manual_action)

        help_action = QAction("Online Manual...", parent=self)
        help_action.triggered.connect(self.show_online_manual)
        help_action.setIcon(ui_helpers.get_icon_for_menu("language", size=24, fill=False))
        action_manager.add_action_to_menu(MenuID.HELP, help_action)

        action_manager.add_action_to_menu(MenuID.HELP, ui_helpers.menu_separator(self))

        show_about_dlg_action = QAction("About PySoWorks", parent=self)
        show_about_dlg_action.triggered.connect(self.show_about_dialog)
        show_about_dlg_action.setIcon(QIcon(str(ui_helpers.images_path() / "app_icon.svg")))
        action_manager.add_action_to_menu(MenuID.HELP, show_about_dlg_action)

    def init_style_controls(self):
        """
        Initializes the style controls for the main window.
        This method sets up the UI elements related to styling and appearance.  
        """
        menu = self.ui.menuView
        menu.addSeparator()
        a = self.light_theme_action = QAction("Light Theme", self)
        a.setCheckable(True)
        a.setChecked(not style_manager.style.is_current_theme_dark())
        menu.addAction(a)
        a.triggered.connect(style_manager.set_light_theme)
        

    def add_view(self, widget_class, title) -> QtAds.CDockWidget:
        """
        Adds a new view to the main window.
        :param widget_class: The class of the widget to be added.
        :param title: The title of the dock widget.
        """
        widget = widget_class(self)
        dock_widget = QtAds.CDockWidget(title)
        dock_widget.setWidget(widget)
        dock_widget.setFeature(QtAds.CDockWidget.DockWidgetDeleteOnClose, True)
        self.dock_manager.addDockWidgetTab(QtAds.CenterDockWidgetArea, dock_widget)
        widget.status_message.connect(self.show_status_message)
        dock_widget.closed.connect(widget.cleanup)
        return dock_widget


    def add_nv200_view(self):
        """
        Adds a new NV200 view to the main window.
        """
        dock_widget = self.add_view(NV200Widget, "NV200")

    def add_spibox_view(self):
        """
        Adds a new SpiBox view to the main window.
        """
        self.add_view(SpiBoxWidget, "SpiBox")


    def show_status_message(self, message: str, timeout: int | None = 4000):
        """
        Displays a status message in the status bar.
        :param message: The message to display.
        """
        if message.startswith("Error"):
            self.statusBar().setStyleSheet("QStatusBar { color: red; }")
        else:
            self.statusBar().setStyleSheet("")
        self.statusBar().showMessage(message, timeout)


    def show_online_manual(self):
        """
        Opens the online manual in the default web browser.
        """
        QDesktopServices.openUrl(QUrl("https://piezosystemjena.github.io/PySoWorks"))

    def show_manual(self):
        """
        Attempts to open the user manual in the default web browser.

        The method first tries to locate the manual in the 'doc/index.html' directory
        relative to the application's executable path. If unsuccessful, it attempts to
        open the manual from 'doc/_build/index.html' relative to the source code location.

        Status messages are displayed with the resolved manual path for debugging purposes.
        """
        base_path = os.path.dirname(sys.executable)
        manual_path = os.path.join(base_path, "doc/index.html")
        if QDesktopServices.openUrl(QUrl.fromLocalFile(manual_path)):
            return
        base_path = Path(__file__).resolve().parent.parent
        manual_path = base_path / "doc/_build/index.html"
        QDesktopServices.openUrl(QUrl.fromLocalFile(manual_path))


    def show_about_dialog(self):
        """
        Displays the About dialog.
        """
        dialog = AboutDialog(self)
        dialog.exec_()

   
def setup_logging():
    """
    Configures the logging settings for the application.
    """
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s.%(msecs)03d | %(name)-25s | %(message)s',
        datefmt="[%X]",
        handlers=[RichHandler(rich_tracebacks=True, )]
    )
    install_rich_traceback(show_locals=True)  

    logging.getLogger("nv200.device_discovery").setLevel(logging.DEBUG)
    logging.getLogger("nv200.transport_protocols").setLevel(logging.DEBUG)         
    logging.getLogger("nv200.serial_protocol").setLevel(logging.DEBUG)    
    logging.getLogger("nv200.device_base").setLevel(logging.DEBUG)     


def resource_path(relative_path):
    """Get absolute path to resource, works for dev and for PyInstaller"""
    base_path = ''
    if getattr(sys, 'frozen', False):
        base_path = sys._MEIPASS
    else:
        base_path = Path(__file__).resolve().parent.parent
    print(f"base_path: {base_path}")
    return os.path.join(base_path, relative_path)


class ExceptionCatchingApplication(QApplication):
    """
    A custom QApplication subclass that catches all exceptions occurring during event notification.

    Overrides the `notify` method to wrap event processing in a try-except block. If an exception is raised while processing an event, it is handled by `ui_helpers.default_exception_handler`, and the method returns False to indicate the event was not handled successfully.

    This helps prevent unhandled exceptions from crashing the application and provides a centralized place for exception handling in the Qt event loop.

    Methods
    -------
    notify(receiver, event)
        Processes the event for the given receiver, catching and handling any exceptions that occur.
    """
    def __init__(self, argv: list[str]) -> None:
        """
        Initialize the application and set the global exception hook.
        """
        super().__init__(argv)
        sys.excepthook = self.handle_exception


    def notify(self, receiver, event):
        """
        Overrides QApplication.notify to catch exceptions during event delivery.
        """
        try:
            return super().notify(receiver, event)
        except Exception as e:
            ui_helpers.default_exception_handler(e)
            return False
        
    def handle_exception(self, exc_type, exc_value, exc_traceback):
        """
        Central handler for uncaught exceptions.
        """
        ui_helpers.default_exception_handler(exc_value)
        # Optional: also print the traceback
        import traceback
        traceback.print_exception(exc_type, exc_value, exc_traceback)



def main():
    """
    Initializes and runs the main application window.
    """
    QLocale.setDefault(QLocale(QLocale.English, QLocale.UnitedStates))
    setup_logging()
    QGuiApplication.setAttribute(Qt.ApplicationAttribute.AA_EnableHighDpiScaling)
    QGuiApplication.setAttribute(Qt.ApplicationAttribute.AA_UseHighDpiPixmaps)
    QApplication.setDesktopSettingsAware(True)

    QApplication.setEffectEnabled(Qt.UIEffect.UI_AnimateMenu, False)
    QApplication.setEffectEnabled(Qt.UIEffect.UI_AnimateCombo, False)

    app = ExceptionCatchingApplication(sys.argv)
    app.setApplicationName('PySoWorks')
    app.setOrganizationName('piezosystem jena')
    app.setOrganizationDomain('https://www.piezosystem.com/')
    version = Path(resource_path("VERSION")).read_text(encoding="utf-8").strip()
    app.setApplicationVersion(version)
    print(f"PySoWorks Version: {version}")
    app.setApplicationDisplayName(f'PySoWorks {version}')
    app_path = Path(__file__).resolve().parent
    print(f"Application Path: {app_path}")
    app.setWindowIcon(QIcon(resource_path('pysoworks/assets/app_icon.ico')))
    style_manager.load_theme_from_settings()

    widget = MainWindow()
    widget.show()
    widget.setWindowTitle(app.applicationDisplayName())

    style_manager.notify_application()

    with qtinter.using_asyncio_from_qt():
        app.exec()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'mainwindow.ui'
##
## Created by: Qt User Interface Compiler version 6.9.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QMainWindow, QMenu, QMenuBar,
    QSizePolicy, QStatusBar, QVBoxLayout, QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1287, 666)
        self.actionNv200View = QAction(MainWindow)
        self.actionNv200View.setObjectName(u"actionNv200View")
        self.actionSpiBoxView = QAction(MainWindow)
        self.actionSpiBoxView.setObjectName(u"actionSpiBoxView")
        self.actionTestView = QAction(MainWindow)
        self.actionTestView.setObjectName(u"actionTestView")
        self.actionBrowseParameterBackups = QAction(MainWindow)
        self.actionBrowseParameterBackups.setObjectName(u"actionBrowseParameterBackups")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout_5 = QVBoxLayout(self.centralwidget)
        self.verticalLayout_5.setSpacing(0)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 1287, 33))
        self.menuView = QMenu(self.menubar)
        self.menuView.setObjectName(u"menuView")
        self.menuHelp = QMenu(self.menubar)
        self.menuHelp.setObjectName(u"menuHelp")
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())
        self.menuView.addAction(self.actionNv200View)
        self.menuView.addAction(self.actionSpiBoxView)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.actionNv200View.setText(QCoreApplication.translate("MainWindow", u"Add NV200 View...", None))
        self.actionSpiBoxView.setText(QCoreApplication.translate("MainWindow", u"Add SpiBox View...", None))
        self.actionTestView.setText(QCoreApplication.translate("MainWindow", u"Add Test View", None))
        self.actionBrowseParameterBackups.setText(QCoreApplication.translate("MainWindow", u"Browse Device Parameter Backups ...", None))
        self.menuView.setTitle(QCoreApplication.translate("MainWindow", u"View", None))
        self.menuHelp.setTitle(QCoreApplication.translate("MainWindow", u"Help", None))
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
    # retranslateUi




import qtass
from typing import cast
from pathlib import Path
from PySide6.QtWidgets import QApplication
from PySide6.QtCore import QStandardPaths
from pysoworks.settings_manager import SettingsContext

class StyleManager:
    """
    A global action manager to decouple widgets from the main window.
    Allows widgets to register actions without directly importing or
    referencing the QMainWindow, avoiding circular imports.
    """

    def __init__(self) -> None:
        QApplication.setStyle('Fusion')
        self.style = qtass.QtAdvancedStylesheet()
        app_path = Path(__file__).resolve().parent
        style = self.style
        style.output_dir = QStandardPaths.writableLocation(QStandardPaths.StandardLocation.AppDataLocation) + "/styles"
        style.set_styles_dir_path(app_path / 'styles')
        style.set_current_style("metro")
        self.dark_mode = True
        self.style.set_current_theme("dark_piezosystem")


    def __set_light_theme(self, light: bool) -> None:
        """
        Sets the light theme for the application.

        Args:
            light (bool): If True, sets the light theme; otherwise, sets the dark theme.
        """
        print(f"Setting light theme: {light}")
        if light:
            self.style.set_current_theme("light_piezosystem")
        else:
            self.style.set_current_theme("dark_piezosystem")
        self.dark_mode = not light
        self.style.update_stylesheet()
        app = QApplication.instance()
        if isinstance(app, QApplication):
            app.setStyleSheet(self.style.stylesheet)


    def apply_stylesheet(self, app: QApplication) -> None:
        """
        Applies the QtAss stylesheet to the given QApplication instance.
        
        Args:
            app (QApplication): The application instance to apply the stylesheet to.
        """
        style = self.style
        style.update_stylesheet()
        app.setStyleSheet(self.style.stylesheet)


    def set_light_theme(self, light: bool) -> None:
        """
        Sets the light theme for the application.

        Args:
            light (bool): If True, sets the light theme; otherwise, sets the dark theme.
        """
        self.__set_light_theme(light)
        with SettingsContext() as settings:
            settings.setValue("theme/light", light)


    def load_theme_from_settings(self) -> None:
        """
        Loads the theme settings from the QSettings.
        Call this function, if all application settings like organization and application name are set.
        """
        with SettingsContext() as settings:
            light_theme = cast(bool, settings.value("theme/light", False, type=bool))
        self.set_light_theme(light_theme)


    def notify_application(self) -> None:
        """
        Notifies the application about style changes.
        This allows existing controls to adapt to dark / or light mode
        """
        self.style.dark_mode_changed.emit(self.style.is_current_theme_dark())


# Global instance of the StyleManager   
style_manager = StyleManager()